@@ -3,7 +3,7 @@ package msg
 import (
 	"fmt"
 
-	"git.sr.ht/~sircmpwn/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~sircmpwn/getopt"
 )
 
@@ -14,7 +14,7 @@ func init() {
 }
 
 func (Mark) Aliases() []string {
-	return []string{"mark", "unmark"}
+	return []string{"mark", "unmark", "remark"}
 }
 
 func (Mark) Complete(aerc *widgets.Aerc, args []string) []string {
@@ -31,24 +31,44 @@ func (Mark) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	opts, _, err := getopt.Getopts(args, "atv")
+	marker := store.Marker()
+	opts, _, err := getopt.Getopts(args, "atvVT")
 	if err != nil {
 		return err
 	}
 	var all bool
 	var toggle bool
 	var visual bool
+	var clearVisual bool
+	var thread bool
 	for _, opt := range opts {
 		switch opt.Option {
 		case 'a':
 			all = true
 		case 'v':
 			visual = true
+			clearVisual = true
+		case 'V':
+			visual = true
 		case 't':
 			toggle = true
+		case 'T':
+			thread = true
 		}
 	}
 
+	if thread && len(store.Threads()) == 0 {
+		return fmt.Errorf("No threads found")
+	}
+
+	if thread && all {
+		return fmt.Errorf("-a and -T are mutually exclusive")
+	}
+
+	if thread && visual {
+		return fmt.Errorf("-v and -T are mutually exclusive")
+	}
+
 	switch args[0] {
 	case "mark":
 		if all && visual {
@@ -57,21 +77,28 @@ func (Mark) Execute(aerc *widgets.Aerc, args []string) error {
 
 		var modFunc func(uint32)
 		if toggle {
-			modFunc = store.ToggleMark
+			modFunc = marker.ToggleMark
 		} else {
-			modFunc = store.Mark
+			modFunc = marker.Mark
 		}
-		if all {
+		switch {
+		case all:
 			uids := store.Uids()
 			for _, uid := range uids {
 				modFunc(uid)
 			}
 			return nil
-		} else if visual {
-			store.ToggleVisualMark()
+		case visual:
+			marker.ToggleVisualMark(clearVisual)
 			return nil
-		} else {
-			modFunc(selected.Uid)
+		default:
+			if thread {
+				for _, uid := range store.SelectedThread().Root().Uids() {
+					modFunc(uid)
+				}
+			} else {
+				modFunc(selected.Uid)
+			}
 			return nil
 		}
 
@@ -80,19 +107,32 @@ func (Mark) Execute(aerc *widgets.Aerc, args []string) error {
 			return fmt.Errorf("visual mode not supported for this command")
 		}
 
-		if all && toggle {
+		switch {
+		case all && toggle:
 			uids := store.Uids()
 			for _, uid := range uids {
-				store.ToggleMark(uid)
+				marker.ToggleMark(uid)
 			}
 			return nil
-		} else if all && !toggle {
-			store.ClearVisualMark()
+		case all && !toggle:
+			marker.ClearVisualMark()
 			return nil
-		} else {
-			store.Unmark(selected.Uid)
+		default:
+			if thread {
+				for _, uid := range store.SelectedThread().Root().Uids() {
+					marker.Unmark(uid)
+				}
+			} else {
+				marker.Unmark(selected.Uid)
+			}
 			return nil
 		}
+	case "remark":
+		if all || visual || toggle || thread {
+			return fmt.Errorf("Usage: :remark")
+		}
+		marker.Remark()
+		return nil
 	}
 	return nil // never reached
 }
