@@ -5,12 +5,15 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"regexp"
 	"strings"
 
 	"git.sr.ht/~sircmpwn/getopt"
 
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
@@ -54,7 +57,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 	}
 
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	acct := widget.SelectedAccount()
 
 	if acct == nil {
@@ -112,6 +115,19 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 	} else {
 		to = msg.Envelope.From
 	}
+
+	if !aerc.Config().Compose.ReplyToSelf {
+		for i, v := range to {
+			if v.Address == from.Address {
+				to = append(to[:i], to[i+1:]...)
+				break
+			}
+		}
+		if len(to) == 0 {
+			to = msg.Envelope.To
+		}
+	}
+
 	recSet.AddList(to)
 
 	if replyAll {
@@ -132,7 +148,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		to = append(to, envTos...)
 
 		for _, addr := range msg.Envelope.Cc {
-			//dedupe stuff from the to/from headers
+			// dedupe stuff from the to/from headers
 			if recSet.Contains(addr) {
 				continue
 			}
@@ -141,12 +157,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		recSet.AddList(cc)
 	}
 
-	var subject string
-	if !strings.HasPrefix(strings.ToLower(msg.Envelope.Subject), "re: ") {
-		subject = "Re: " + msg.Envelope.Subject
-	} else {
-		subject = msg.Envelope.Subject
-	}
+	subject := "Re: " + trimLocalizedRe(msg.Envelope.Subject)
 
 	h := &mail.Header{}
 	h.SetAddressList("to", to)
@@ -200,6 +211,31 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 			template = aerc.Config().Templates.QuotedReply
 		}
 
+		if crypto.IsEncrypted(msg.BodyStructure) {
+			provider := aerc.SelectedTabContent().(widgets.ProvidesMessage)
+			mv, ok := provider.(*widgets.MessageViewer)
+			if !ok {
+				return fmt.Errorf("message is encrypted. can only quote reply while message is open")
+			}
+			p := provider.SelectedMessagePart()
+			if p == nil {
+				return fmt.Errorf("could not fetch message part")
+			}
+			mv.MessageView().FetchBodyPart(p.Index, func(reader io.Reader) {
+				buf := new(bytes.Buffer)
+				_, err := buf.ReadFrom(reader)
+				if err != nil {
+					logging.Warnf("failed to fetch bodypart: %v", err)
+				}
+				original.Text = buf.String()
+				err = addTab()
+				if err != nil {
+					logging.Warnf("failed to add tab: %v", err)
+				}
+			})
+			return nil
+		}
+
 		part := lib.FindPlaintext(msg.BodyStructure, nil)
 		if part == nil {
 			// mkey... let's get the first thing that isn't a container
@@ -215,12 +251,21 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 
 		store.FetchBodyPart(msg.Uid, part, func(reader io.Reader) {
 			buf := new(bytes.Buffer)
-			buf.ReadFrom(reader)
+			_, err := buf.ReadFrom(reader)
+			if err != nil {
+				logging.Warnf("failed to fetch bodypart: %v", err)
+			}
 			original.Text = buf.String()
-			addTab()
+			err = addTab()
+			if err != nil {
+				logging.Warnf("failed to add tab: %v", err)
+			}
 		})
 		return nil
 	} else {
+		if template == "" {
+			template = aerc.Config().Templates.NewMessage
+		}
 		return addTab()
 	}
 }
@@ -247,8 +292,8 @@ func (s addrSet) Contains(a *mail.Address) bool {
 	return ok
 }
 
-//setReferencesHeader adds the references header to target based on parent
-//according to RFC2822
+// setReferencesHeader adds the references header to target based on parent
+// according to RFC2822
 func setReferencesHeader(target, parent *mail.Header) error {
 	refs, err := parent.MsgIDList("references")
 	if err != nil {
@@ -273,7 +318,8 @@ func setReferencesHeader(target, parent *mail.Header) error {
 
 // addMimeType adds the proper mime type of the part to the originalMail struct
 func addMimeType(msg *models.MessageInfo, part []int,
-	orig *models.OriginalMail) error {
+	orig *models.OriginalMail,
+) error {
 	// caution, :forward uses the code as well, keep that in mind when modifying
 	bs, err := msg.BodyStructure.PartAtIndex(part)
 	if err != nil {
@@ -282,3 +328,11 @@ func addMimeType(msg *models.MessageInfo, part []int,
 	orig.MIMEType = fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
 	return nil
 }
+
+// trimLocalizedRe removes known localizations of Re: commonly used by Outlook.
+func trimLocalizedRe(subject string) string {
+	return strings.TrimPrefix(subject, localizedRe.FindString(subject))
+}
+
+// localizedRe contains a list of known translations for the common Re:
+var localizedRe = regexp.MustCompile(`(?i)^((AW|RE|SV|VS|ODP|R): ?)+`)
