@@ -9,6 +9,7 @@ import (
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
@@ -37,7 +38,7 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(args) != 1 {
 		return errors.New("Usage: unsubscribe")
 	}
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	msg, err := widget.SelectedMessage()
 	if err != nil {
 		return err
@@ -54,10 +55,10 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(methods) == 0 {
 		return fmt.Errorf("no methods found to unsubscribe")
 	}
-	aerc.Logger().Printf("unsubscribe: found %d methods", len(methods))
+	logging.Infof("unsubscribe: found %d methods", len(methods))
 
 	unsubscribe := func(method *url.URL) {
-		aerc.Logger().Printf("unsubscribe: trying to unsubscribe using %v", method.Scheme)
+		logging.Infof("unsubscribe: trying to unsubscribe using %s", method.Scheme)
 		var err error
 		switch strings.ToLower(method.Scheme) {
 		case "mailto":
@@ -104,7 +105,6 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 				}
 			}
 			aerc.PushError("Unsubscribe: selected method not found")
-			return
 		},
 	)
 	aerc.AddDialog(dialog)
@@ -135,7 +135,7 @@ func parseUnsubscribeMethods(header string) (methods []*url.URL) {
 }
 
 func unsubscribeMailto(aerc *widgets.Aerc, u *url.URL) error {
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	acct := widget.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
@@ -179,17 +179,16 @@ func unsubscribeHTTP(aerc *widgets.Aerc, u *url.URL) error {
 		"Do you want to open this link?",
 		u.String(),
 		[]string{"No", "Yes"}, 0, aerc.SelectedAccountUiConfig(),
-		func(option string, err error) {
+		func(option string, _ error) {
 			aerc.CloseDialog()
 			switch option {
 			case "Yes":
-				if err = lib.NewXDGOpen(u.String()).Start(); err != nil {
+				if err := lib.NewXDGOpen(u.String()).Start(); err != nil {
 					aerc.PushError("Unsubscribe:" + err.Error())
 				}
 			default:
 				aerc.PushError("Unsubscribe: link will not be opened")
 			}
-			return
 		},
 	)
 	aerc.AddDialog(confirm)
