@@ -3,7 +3,7 @@ package msgview
 import (
 	"errors"
 
-	"git.sr.ht/~sircmpwn/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
 type Close struct{}
@@ -24,7 +24,7 @@ func (Close) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(args) != 1 {
 		return errors.New("Usage: close")
 	}
-	mv, _ := aerc.SelectedTab().(*widgets.MessageViewer)
+	mv, _ := aerc.SelectedTabContent().(*widgets.MessageViewer)
 	mv.Close()
 	aerc.RemoveTab(mv)
 	return nil
