@@ -22,7 +22,7 @@ type Binding struct {
 }
 
 type KeyBindings struct {
-	bindings []*Binding
+	Bindings []*Binding
 
 	// If false, disable global keybindings in this context
 	Globals bool
@@ -48,7 +48,7 @@ func NewKeyBindings() *KeyBindings {
 func MergeBindings(bindings ...*KeyBindings) *KeyBindings {
 	merged := NewKeyBindings()
 	for _, b := range bindings {
-		merged.bindings = append(merged.bindings, b.bindings...)
+		merged.Bindings = append(merged.Bindings, b.Bindings...)
 	}
 	merged.ExKey = bindings[0].ExKey
 	merged.Globals = bindings[0].Globals
@@ -56,8 +56,8 @@ func MergeBindings(bindings ...*KeyBindings) *KeyBindings {
 }
 
 func (config AercConfig) MergeContextualBinds(baseBinds *KeyBindings,
-	contextType ContextType, reg string, bindCtx string) *KeyBindings {
-
+	contextType ContextType, reg string, bindCtx string,
+) *KeyBindings {
 	bindings := baseBinds
 	for _, contextualBind := range config.ContextualBinds {
 		if contextualBind.ContextType != contextType {
@@ -79,16 +79,16 @@ func (config AercConfig) MergeContextualBinds(baseBinds *KeyBindings,
 
 func (bindings *KeyBindings) Add(binding *Binding) {
 	// TODO: Search for conflicts?
-	bindings.bindings = append(bindings.bindings, binding)
+	bindings.Bindings = append(bindings.Bindings, binding)
 }
 
 func (bindings *KeyBindings) GetBinding(
-	input []KeyStroke) (BindingSearchResult, []KeyStroke) {
-
+	input []KeyStroke,
+) (BindingSearchResult, []KeyStroke) {
 	incomplete := false
 	// TODO: This could probably be a sorted list to speed things up
 	// TODO: Deal with bindings that share a prefix
-	for _, binding := range bindings.bindings {
+	for _, binding := range bindings.Bindings {
 		if len(binding.Input) < len(input) {
 			continue
 		}
@@ -121,7 +121,7 @@ func (bindings *KeyBindings) GetBinding(
 func (bindings *KeyBindings) GetReverseBindings(output []KeyStroke) [][]KeyStroke {
 	var inputs [][]KeyStroke
 
-	for _, binding := range bindings.bindings {
+	for _, binding := range bindings.Bindings {
 		if len(binding.Output) != len(output) {
 			continue
 		}
@@ -149,8 +149,11 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 		s := ""
 		for name, ks := range keyNames {
 			if ks.Modifiers == stroke.Modifiers && ks.Key == stroke.Key && ks.Rune == stroke.Rune {
-				if name == "cr" {
+				switch name {
+				case "cr", "c-m":
 					name = "enter"
+				case "c-i":
+					name = "tab"
 				}
 				s = fmt.Sprintf("<%s>", name)
 				break
@@ -165,9 +168,7 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 	return sb.String()
 }
 
-var (
-	keyNames map[string]KeyStroke
-)
+var keyNames map[string]KeyStroke
 
 func ParseKeyStrokes(keystrokes string) ([]KeyStroke, error) {
 	var strokes []KeyStroke
@@ -184,11 +185,12 @@ func ParseKeyStrokes(keystrokes string) ([]KeyStroke, error) {
 		switch tok {
 		case '<':
 			name, err := buf.ReadString(byte('>'))
-			if err == io.EOF {
+			switch {
+			case err == io.EOF:
 				return nil, errors.New("Expecting '>'")
-			} else if err != nil {
+			case err != nil:
 				return nil, err
-			} else if name == ">" {
+			case name == ">":
 				return nil, errors.New("Expected a key name")
 			}
 			name = name[:len(name)-1]
@@ -340,11 +342,11 @@ func init() {
 	keyNames["c-f"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlF, 0}
 	keyNames["c-g"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlG, 0}
 	keyNames["c-h"] = KeyStroke{tcell.ModNone, tcell.KeyCtrlH, 0}
-	keyNames["c-i"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlI, 0}
+	keyNames["c-i"] = KeyStroke{tcell.ModNone, tcell.KeyCtrlI, 0}
 	keyNames["c-j"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlJ, 0}
 	keyNames["c-k"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlK, 0}
 	keyNames["c-l"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlL, 0}
-	keyNames["c-m"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlM, 0}
+	keyNames["c-m"] = KeyStroke{tcell.ModNone, tcell.KeyCtrlM, 0}
 	keyNames["c-n"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlN, 0}
 	keyNames["c-o"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlO, 0}
 	keyNames["c-p"] = KeyStroke{tcell.ModCtrl, tcell.KeyCtrlP, 0}
