@@ -3,7 +3,6 @@ package config
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"net/url"
 	"os"
@@ -23,6 +22,7 @@ import (
 	"github.com/mitchellh/go-homedir"
 
 	"git.sr.ht/~rjarry/aerc/lib/templates"
+	"git.sr.ht/~rjarry/aerc/logging"
 )
 
 type GeneralConfig struct {
@@ -47,6 +47,7 @@ type UIConfig struct {
 	MouseEnabled        bool          `ini:"mouse-enabled"`
 	ThreadingEnabled    bool          `ini:"threading-enabled"`
 	ForceClientThreads  bool          `ini:"force-client-threads"`
+	ClientThreadsDelay  time.Duration `ini:"client-threads-delay"`
 	FuzzyComplete       bool          `ini:"fuzzy-complete"`
 	NewMessageBell      bool          `ini:"new-message-bell"`
 	Spinner             string        `ini:"spinner"`
@@ -60,6 +61,7 @@ type UIConfig struct {
 	DirListFormat       string        `ini:"dirlist-format"`
 	DirListDelay        time.Duration `ini:"dirlist-delay"`
 	DirListTree         bool          `ini:"dirlist-tree"`
+	DirListCollapse     int           `ini:"dirlist-collapse"`
 	Sort                []string      `delim:" "`
 	NextMessageOnDelete bool          `ini:"next-message-on-delete"`
 	CompletionDelay     time.Duration `ini:"completion-delay"`
@@ -93,6 +95,49 @@ const (
 	FILTER_HEADER
 )
 
+type RemoteConfig struct {
+	Value       string
+	PasswordCmd string
+}
+
+func (c RemoteConfig) parseValue() (*url.URL, error) {
+	return url.Parse(c.Value)
+}
+
+func (c RemoteConfig) ConnectionString() (string, error) {
+	if c.Value == "" || c.PasswordCmd == "" {
+		return c.Value, nil
+	}
+
+	u, err := c.parseValue()
+	if err != nil {
+		return "", err
+	}
+
+	// ignore the command if a password is specified
+	if _, exists := u.User.Password(); exists {
+		return c.Value, nil
+	}
+
+	// don't attempt to parse the command if the url is a path (ie /usr/bin/sendmail)
+	if !u.IsAbs() {
+		return c.Value, nil
+	}
+
+	cmd := exec.Command("sh", "-c", c.PasswordCmd)
+	cmd.Stdin = os.Stdin
+	output, err := cmd.Output()
+	if err != nil {
+		return "", fmt.Errorf("failed to read password: %w", err)
+	}
+
+	pw := strings.TrimSpace(string(output))
+	u.User = url.UserPassword(u.User.Username(), pw)
+	c.Value = u.String()
+
+	return c.Value, nil
+}
+
 type AccountConfig struct {
 	Archive           string
 	CopyTo            string
@@ -102,16 +147,15 @@ type AccountConfig struct {
 	Aliases           string
 	Name              string
 	Source            string
-	SourceCredCmd     string
 	Folders           []string
 	FoldersExclude    []string
 	Params            map[string]string
-	Outgoing          string
-	OutgoingCredCmd   string
+	Outgoing          RemoteConfig
 	SignatureFile     string
 	SignatureCmd      string
 	EnableFoldersSort bool     `ini:"enable-folders-sort"`
 	FoldersSort       []string `ini:"folders-sort" delim:","`
+	AddressBookCmd    string   `ini:"address-book-cmd"`
 
 	// CheckMail
 	CheckMail        time.Duration `ini:"check-mail"`
@@ -222,7 +266,7 @@ func mapName(raw string) string {
 	return string(newstr)
 }
 
-func loadAccountConfig(path string) ([]AccountConfig, error) {
+func loadAccountConfig(path string, accts []string) ([]AccountConfig, error) {
 	file, err := ini.Load(path)
 	if err != nil {
 		// No config triggers account configuration wizard
@@ -235,7 +279,11 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 		if _sec == "DEFAULT" {
 			continue
 		}
+		if len(accts) > 0 && !contains(accts, _sec) {
+			continue
+		}
 		sec := file.Section(_sec)
+		sourceRemoteConfig := RemoteConfig{}
 		account := AccountConfig{
 			Archive:           "Archive",
 			Default:           "INBOX",
@@ -249,38 +297,45 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 			return nil, err
 		}
 		for key, val := range sec.KeysHash() {
-			if key == "folders" {
+			switch key {
+			case "folders":
 				folders := strings.Split(val, ",")
 				sort.Strings(folders)
 				account.Folders = folders
-			} else if key == "folders-exclude" {
+			case "folders-exclude":
 				folders := strings.Split(val, ",")
 				sort.Strings(folders)
 				account.FoldersExclude = folders
-			} else if key == "source-cred-cmd" {
-				account.SourceCredCmd = val
-			} else if key == "outgoing" {
-				account.Outgoing = val
-			} else if key == "outgoing-cred-cmd" {
-				account.OutgoingCredCmd = val
-			} else if key == "from" {
+			case "source":
+				sourceRemoteConfig.Value = val
+			case "source-cred-cmd":
+				sourceRemoteConfig.PasswordCmd = val
+			case "outgoing":
+				account.Outgoing.Value = val
+			case "outgoing-cred-cmd":
+				account.Outgoing.PasswordCmd = val
+			case "from":
 				account.From = val
-			} else if key == "aliases" {
+			case "aliases":
 				account.Aliases = val
-			} else if key == "copy-to" {
+			case "copy-to":
 				account.CopyTo = val
-			} else if key == "archive" {
+			case "archive":
 				account.Archive = val
-			} else if key == "enable-folders-sort" {
+			case "enable-folders-sort":
 				account.EnableFoldersSort, _ = strconv.ParseBool(val)
-			} else if key == "pgp-key-id" {
+			case "pgp-key-id":
 				account.PgpKeyId = val
-			} else if key == "pgp-auto-sign" {
+			case "pgp-auto-sign":
 				account.PgpAutoSign, _ = strconv.ParseBool(val)
-			} else if key == "pgp-opportunistic-encrypt" {
+			case "pgp-opportunistic-encrypt":
 				account.PgpOpportunisticEncrypt, _ = strconv.ParseBool(val)
-			} else if key != "name" {
-				account.Params[key] = val
+			case "address-book-cmd":
+				account.AddressBookCmd = val
+			default:
+				if key != "name" {
+					account.Params[key] = val
+				}
 			}
 		}
 		if account.Source == "" {
@@ -290,56 +345,32 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 			return nil, fmt.Errorf("Expected from for account %s", _sec)
 		}
 
-		source, err := parseCredential(account.Source, account.SourceCredCmd)
+		source, err := sourceRemoteConfig.ConnectionString()
 		if err != nil {
-			return nil, fmt.Errorf("Invalid source credentials for %s: %s", _sec, err)
+			return nil, fmt.Errorf("Invalid source credentials for %s: %w", _sec, err)
 		}
 		account.Source = source
 
-		outgoing, err := parseCredential(account.Outgoing, account.OutgoingCredCmd)
+		_, err = account.Outgoing.parseValue()
 		if err != nil {
-			return nil, fmt.Errorf("Invalid outgoing credentials for %s: %s", _sec, err)
+			return nil, fmt.Errorf("Invalid outgoing credentials for %s: %w", _sec, err)
 		}
-		account.Outgoing = outgoing
 
 		accounts = append(accounts, account)
 	}
+	if len(accts) > 0 {
+		// Sort accounts struct to match the specified order, if we
+		// have one
+		if len(accounts) != len(accts) {
+			return nil, errors.New("account(s) not found")
+		}
+		sort.Slice(accounts, func(i, j int) bool {
+			return accts[i] < accts[j]
+		})
+	}
 	return accounts, nil
 }
 
-func parseCredential(cred, command string) (string, error) {
-	if cred == "" || command == "" {
-		return cred, nil
-	}
-
-	u, err := url.Parse(cred)
-	if err != nil {
-		return "", err
-	}
-
-	// ignore the command if a password is specified
-	if _, exists := u.User.Password(); exists {
-		return cred, nil
-	}
-
-	// don't attempt to parse the command if the url is a path (ie /usr/bin/sendmail)
-	if !u.IsAbs() {
-		return cred, nil
-	}
-
-	cmd := exec.Command("sh", "-c", command)
-	cmd.Stdin = os.Stdin
-	output, err := cmd.Output()
-	if err != nil {
-		return "", fmt.Errorf("failed to read password: %s", err)
-	}
-
-	pw := strings.TrimSpace(string(output))
-	u.User = url.UserPassword(u.User.Username(), pw)
-
-	return u.String(), nil
-}
-
 // Set at build time
 var shareDir string
 
@@ -382,14 +413,14 @@ var searchDirs = buildDefaultDirs()
 func installTemplate(root, name string) error {
 	var err error
 	if _, err = os.Stat(root); os.IsNotExist(err) {
-		err = os.MkdirAll(root, 0755)
+		err = os.MkdirAll(root, 0o755)
 		if err != nil {
 			return err
 		}
 	}
 	var data []byte
 	for _, dir := range searchDirs {
-		data, err = ioutil.ReadFile(path.Join(dir, name))
+		data, err = os.ReadFile(path.Join(dir, name))
 		if err == nil {
 			break
 		}
@@ -397,7 +428,7 @@ func installTemplate(root, name string) error {
 	if err != nil {
 		return err
 	}
-	err = ioutil.WriteFile(path.Join(root, name), data, 0644)
+	err = os.WriteFile(path.Join(root, name), data, 0o644)
 	if err != nil {
 		return err
 	}
@@ -413,25 +444,26 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 				Command: cmd,
 				Filter:  match,
 			}
-			if strings.Contains(match, ",~") {
+			switch {
+			case strings.Contains(match, ",~"):
 				filter.FilterType = FILTER_HEADER
-				header := filter.Filter[:strings.Index(filter.Filter, ",")]
+				header := filter.Filter[:strings.Index(filter.Filter, ",")] //nolint:gocritic // guarded by strings.Contains
 				regex := filter.Filter[strings.Index(filter.Filter, "~")+1:]
 				filter.Header = strings.ToLower(header)
 				filter.Regex, err = regexp.Compile(regex)
 				if err != nil {
 					return err
 				}
-			} else if strings.ContainsRune(match, ',') {
+			case strings.ContainsRune(match, ','):
 				filter.FilterType = FILTER_HEADER
-				header := filter.Filter[:strings.Index(filter.Filter, ",")]
+				header := filter.Filter[:strings.Index(filter.Filter, ",")] //nolint:gocritic // guarded by strings.Contains
 				value := filter.Filter[strings.Index(filter.Filter, ",")+1:]
 				filter.Header = strings.ToLower(header)
 				filter.Regex, err = regexp.Compile(regexp.QuoteMeta(value))
 				if err != nil {
 					return err
 				}
-			} else {
+			default:
 				filter.FilterType = FILTER_MIMETYPE
 			}
 			config.Filters = append(config.Filters, filter)
@@ -460,8 +492,7 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 			return err
 		}
 		for key, val := range compose.KeysHash() {
-			switch key {
-			case "header-layout":
+			if key == "header-layout" {
 				config.Compose.HeaderLayout = parseLayout(val)
 			}
 		}
@@ -511,27 +542,27 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 		if err := validateBorderChars(uiSection, &uiSubConfig); err != nil {
 			return err
 		}
-		contextualUi :=
-			UIConfigContext{
-				UiConfig: uiSubConfig,
-			}
+		contextualUi := UIConfigContext{
+			UiConfig: uiSubConfig,
+		}
 
 		var index int
-		if strings.Contains(sectionName, "~") {
+		switch {
+		case strings.Contains(sectionName, "~"):
 			index = strings.Index(sectionName, "~")
 			regex := string(sectionName[index+1:])
 			contextualUi.Regex, err = regexp.Compile(regex)
 			if err != nil {
 				return err
 			}
-		} else if strings.Contains(sectionName, "=") {
+		case strings.Contains(sectionName, "="):
 			index = strings.Index(sectionName, "=")
 			value := string(sectionName[index+1:])
 			contextualUi.Regex, err = regexp.Compile(regexp.QuoteMeta(value))
 			if err != nil {
 				return err
 			}
-		} else {
+		default:
 			return fmt.Errorf("Invalid Ui Context regex in %s", sectionName)
 		}
 
@@ -636,8 +667,7 @@ func validatePgpProvider(section *ini.Section) error {
 		"internal": true,
 	}
 	for key, val := range section.KeysHash() {
-		switch key {
-		case "pgp-provider":
+		if key == "pgp-provider" {
 			if !m[strings.ToLower(val)] {
 				return fmt.Errorf("%v must be either 'gpg' or 'internal'", key)
 			}
@@ -646,7 +676,7 @@ func validatePgpProvider(section *ini.Section) error {
 	return nil
 }
 
-func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
+func LoadConfigFromFile(root *string, accts []string) (*AercConfig, error) {
 	if root == nil {
 		_root := path.Join(xdg.ConfigHome(), "aerc")
 		root = &_root
@@ -655,11 +685,14 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 
 	// if it doesn't exist copy over the template, then load
 	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
+		logging.Debugf("%s not found, installing the system default", filename)
 		if err := installTemplate(*root, "aerc.conf"); err != nil {
 			return nil, err
 		}
 	}
 
+	logging.Infof("Parsing configuration from %s", filename)
+
 	file, err := ini.LoadSources(ini.LoadOptions{
 		KeyValueDelimiters: "=",
 	}, filename)
@@ -705,6 +738,7 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 			EmptyMessage:        "(no messages)",
 			EmptyDirlist:        "(no folders)",
 			MouseEnabled:        false,
+			ClientThreadsDelay:  50 * time.Millisecond,
 			NewMessageBell:      true,
 			FuzzyComplete:       false,
 			Spinner:             "[..]    , [..]   ,  [..]  ,   [..] ,    [..],   [..] ,  [..]  , [..]   ",
@@ -784,6 +818,15 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 		}
 	}
 
+	logging.Debugf("aerc.conf: [general] %#v", config.General)
+	logging.Debugf("aerc.conf: [ui] %#v", config.Ui)
+	logging.Debugf("aerc.conf: [statusline] %#v", config.Statusline)
+	logging.Debugf("aerc.conf: [viewer] %#v", config.Viewer)
+	logging.Debugf("aerc.conf: [compose] %#v", config.Compose)
+	logging.Debugf("aerc.conf: [filters] %#v", config.Filters)
+	logging.Debugf("aerc.conf: [triggers] %#v", config.Triggers)
+	logging.Debugf("aerc.conf: [templates] %#v", config.Templates)
+
 	filename = path.Join(*root, "accounts.conf")
 	if !config.General.UnsafeAccountsConf {
 		if err := checkConfigPerms(filename); err != nil {
@@ -792,21 +835,28 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 	}
 
 	accountsPath := path.Join(*root, "accounts.conf")
-	if accounts, err := loadAccountConfig(accountsPath); err != nil {
+	logging.Infof("Parsing accounts configuration from %s", accountsPath)
+	if accounts, err := loadAccountConfig(accountsPath, accts); err != nil {
 		return nil, err
 	} else {
 		config.Accounts = accounts
 	}
 
+	for _, acct := range config.Accounts {
+		logging.Debugf("accounts.conf: [%s] from = %s", acct.Name, acct.From)
+	}
+
 	filename = path.Join(*root, "binds.conf")
-	binds, err := ini.Load(filename)
-	if err != nil {
+	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
+		logging.Debugf("%s not found, installing the system default", filename)
 		if err := installTemplate(*root, "binds.conf"); err != nil {
 			return nil, err
 		}
-		if binds, err = ini.Load(filename); err != nil {
-			return nil, err
-		}
+	}
+	logging.Infof("Parsing key bindings configuration from %s", filename)
+	binds, err := ini.Load(filename)
+	if err != nil {
+		return nil, err
 	}
 
 	baseGroups := map[string]**KeyBindings{
@@ -823,10 +873,10 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 	// Base Bindings
 	for _, sectionName := range binds.SectionStrings() {
 		// Handle :: delimeter
-		baseSectionName := strings.Replace(sectionName, "::", "////", -1)
+		baseSectionName := strings.ReplaceAll(sectionName, "::", "////")
 		sections := strings.Split(baseSectionName, ":")
 		baseOnly := len(sections) == 1
-		baseSectionName = strings.Replace(sections[0], "////", "::", -1)
+		baseSectionName = strings.ReplaceAll(sections[0], "////", "::")
 
 		group, ok := baseGroups[strings.ToLower(baseSectionName)]
 		if !ok {
@@ -834,7 +884,7 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 		}
 
 		if baseOnly {
-			err = config.LoadBinds(binds, baseSectionName, group, logger)
+			err = config.LoadBinds(binds, baseSectionName, group)
 			if err != nil {
 				return nil, err
 			}
@@ -847,6 +897,7 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 			contextBind.Bindings.Globals = false
 		}
 	}
+	logging.Debugf("binds.conf: %#v", config.Bindings)
 
 	return config, nil
 }
@@ -884,8 +935,7 @@ func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 	return bindings, nil
 }
 
-func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings, logger *log.Logger) error {
-
+func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings) error {
 	if sec, err := binds.GetSection(baseName); err == nil {
 		binds, err := LoadBindingSection(sec)
 		if err != nil {
@@ -910,11 +960,10 @@ func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup
 			return err
 		}
 
-		contextualBind :=
-			BindingConfigContext{
-				Bindings:    binds,
-				BindContext: baseName,
-			}
+		contextualBind := BindingConfigContext{
+			Bindings:    binds,
+			BindContext: baseName,
+		}
 
 		var index int
 		if strings.Contains(sectionName, "=") {
@@ -939,7 +988,7 @@ func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup
 				}
 			}
 			if !valid {
-				logger.Printf("Tried to define binds for unexistent account: %v\n", acctName)
+				logging.Warnf("binds.conf: unexistent account: %s", acctName)
 				continue
 			}
 			contextualBind.ContextType = BIND_CONTEXT_ACCOUNT
@@ -969,7 +1018,7 @@ func checkConfigPerms(filename string) error {
 
 	perms := info.Mode().Perm()
 	// group or others have read access
-	if perms&044 != 0 {
+	if perms&0o44 != 0 {
 		fmt.Fprintf(os.Stderr, "The file %v has too open permissions.\n", filename)
 		fmt.Fprintln(os.Stderr, "This is a security issue (it contains passwords).")
 		fmt.Fprintf(os.Stderr, "To fix it, run `chmod 600 %v`\n", filename)
@@ -991,14 +1040,15 @@ func (ui *UIConfig) loadStyleSet(styleSetDirs []string) error {
 	ui.style = NewStyleSet()
 	err := ui.style.LoadStyleSet(ui.StyleSetName, styleSetDirs)
 	if err != nil {
-		return fmt.Errorf("Unable to load default styleset: %s", err)
+		return fmt.Errorf("Unable to load default styleset: %w", err)
 	}
 
 	return nil
 }
 
 func (config AercConfig) mergeContextualUi(baseUi UIConfig,
-	contextType ContextType, s string) UIConfig {
+	contextType ContextType, s string,
+) UIConfig {
 	for _, contextualUi := range config.ContextualUis {
 		if contextualUi.ContextType != contextType {
 			continue
@@ -1008,7 +1058,10 @@ func (config AercConfig) mergeContextualUi(baseUi UIConfig,
 			continue
 		}
 
-		mergo.Merge(&baseUi, contextualUi.UiConfig, mergo.WithOverride)
+		err := mergo.Merge(&baseUi, contextualUi.UiConfig, mergo.WithOverride)
+		if err != nil {
+			logging.Warnf("merge ui failed: %v", err)
+		}
 		if contextualUi.UiConfig.StyleSetName != "" {
 			baseUi.style = contextualUi.UiConfig.style
 		}
@@ -1041,10 +1094,20 @@ func (uiConfig UIConfig) GetStyleSelected(so StyleObject) tcell.Style {
 }
 
 func (uiConfig UIConfig) GetComposedStyle(base StyleObject,
-	styles []StyleObject) tcell.Style {
+	styles []StyleObject,
+) tcell.Style {
 	return uiConfig.style.Compose(base, styles)
 }
 
 func (uiConfig UIConfig) GetComposedStyleSelected(base StyleObject, styles []StyleObject) tcell.Style {
 	return uiConfig.style.ComposeSelected(base, styles)
 }
+
+func contains(list []string, v string) bool {
+	for _, item := range list {
+		if item == v {
+			return true
+		}
+	}
+	return false
+}
