@@ -9,16 +9,20 @@ read -rp "next tag ($next_tag)? " n
 if [ -n "$n" ]; then
 	next_tag="$n"
 fi
+tag_url="https://git.sr.ht/~rjarry/aerc/refs/$next_tag"
 
-echo "======= Updating version in Makefile..."
+echo "======= Creating release commit..."
 sed -i Makefile -e "s/$prev_tag/$next_tag/g"
-git add Makefile
+sed -i CHANGELOG.md -e "s|^## \[Unreleased\].*|&\n\n## [$next_tag]($tag_url) - $(date +%Y-%m-%d)|"
+git add Makefile CHANGELOG.md
 git commit -sm "Release version $next_tag"
 
 echo "======= Creating tag..."
-git tag --edit --sign \
+changes=$(sed -n "/^## \[$next_tag\].*/,/^## \[$prev_tag\].*/{//!p;}" \
+	CHANGELOG.md | sed '1d;$d;s/^#\+/#/' )
+git -c core.commentchar='%' tag --edit --sign \
 	-m "Release $next_tag highlights:" \
-	-m "$(git log --format='- %s' $prev_tag..)" \
+	-m "$changes" \
 	-m "Thanks to all contributors!" \
 	-m "~\$ git shortlog -sn $prev_tag..$next_tag
 $(git shortlog -sn $prev_tag..)" \
@@ -34,21 +38,23 @@ trap "rm -f -- $email" EXIT
 
 cat >"$email" <<EOF
 To: aerc-annouce <~rjarry/aerc-announce@lists.sr.ht>
-Cc: aerc <~sircmpwn/aerc@lists.sr.ht>
+Cc: aerc-devel <~rjarry/aerc-devel@lists.sr.ht>
+Bcc: aerc <~sircmpwn/aerc@lists.sr.ht>,
+	$(git config user.name) <$(git config user.email)>
 Reply-To: aerc-devel <~rjarry/aerc-devel@lists.sr.ht>
 Subject: aerc $next_tag
 User-Agent: aerc/$next_tag
-Message-ID: <$(date +%Y%m%d%H%M%S).$(base64 -w20 < /dev/urandom | head -n1)@$(hostname)>
+Message-ID: <$(date +%Y%m%d%H%M%S).$(base32 -w12 < /dev/urandom | head -n1)@$(hostname)>
 
 Hi all,
 
 I am glad to announce the release of aerc $next_tag.
 
-https://git.sr.ht/~rjarry/aerc/refs/$next_tag
+$tag_url
 
 $(git tag -l --format='%(contents)' "$next_tag" | sed -n '/BEGIN PGP SIGNATURE/q;p')
 EOF
 
-$EDITOR "$email"
+${EDITOR:-vi} "$email"
 
 /usr/sbin/sendmail -t < "$email"
