@@ -3,8 +3,8 @@ package ui
 import (
 	"fmt"
 
-	"github.com/gdamore/tcell"
-	"github.com/gdamore/tcell/views"
+	"github.com/gdamore/tcell/v2"
+	"github.com/gdamore/tcell/v2/views"
 	"github.com/mattn/go-runewidth"
 )
 
@@ -63,7 +63,8 @@ func (ctx *Context) SetCell(x, y int, ch rune, style tcell.Style) {
 }
 
 func (ctx *Context) Printf(x, y int, style tcell.Style,
-	format string, a ...interface{}) int {
+	format string, a ...interface{},
+) int {
 	width, height := ctx.viewport.Size()
 
 	if x >= width || y >= height {
