@@ -5,6 +5,8 @@ import (
 	"math"
 	"sync"
 	"sync/atomic"
+
+	"github.com/gdamore/tcell/v2"
 )
 
 type Grid struct {
@@ -29,10 +31,12 @@ const (
 type GridSpec struct {
 	// One of SIZE_EXACT or SIZE_WEIGHT
 	Strategy int
-	// If Strategy = SIZE_EXACT, this is the number of cells this row/col shall
-	// occupy. If SIZE_WEIGHT, the space left after all exact rows/cols are
-	// measured is distributed amonst the remainder weighted by this value.
-	Size int
+
+	// If Strategy = SIZE_EXACT, this function returns the number of cells
+	// this row/col shall occupy. If SIZE_WEIGHT, the space left after all
+	// exact rows/cols are measured is distributed amonst the remainder
+	// weighted by the value returned by this function.
+	Size func() int
 }
 
 // Used to cache layout of each row/column
@@ -59,11 +63,11 @@ func NewGrid() *Grid {
 func MakeGrid(numRows, numCols, rowStrategy, colStrategy int) *Grid {
 	rows := make([]GridSpec, numRows)
 	for i := 0; i < numRows; i++ {
-		rows[i] = GridSpec{rowStrategy, 1}
+		rows[i] = GridSpec{rowStrategy, Const(1)}
 	}
 	cols := make([]GridSpec, numCols)
 	for i := 0; i < numCols; i++ {
-		cols[i] = GridSpec{colStrategy, 1}
+		cols[i] = GridSpec{colStrategy, Const(1)}
 	}
 	return NewGrid().Rows(rows).Columns(cols)
 }
@@ -119,6 +123,10 @@ func (grid *Grid) Draw(ctx *Context) {
 		cols := grid.columnLayout[cell.Column : cell.Column+cell.ColSpan]
 		x := cols[0].Offset
 		y := rows[0].Offset
+		if x < 0 || y < 0 {
+			continue
+		}
+
 		width := 0
 		height := 0
 		for _, col := range cols {
@@ -141,6 +149,44 @@ func (grid *Grid) Draw(ctx *Context) {
 	}
 }
 
+func (grid *Grid) MouseEvent(localX int, localY int, event tcell.Event) {
+	if event, ok := event.(*tcell.EventMouse); ok {
+		invalid := grid.invalid
+
+		grid.mutex.RLock()
+		defer grid.mutex.RUnlock()
+
+		for _, cell := range grid.cells {
+			cellInvalid := cell.invalid.Load().(bool)
+			if !cellInvalid && !invalid {
+				continue
+			}
+			rows := grid.rowLayout[cell.Row : cell.Row+cell.RowSpan]
+			cols := grid.columnLayout[cell.Column : cell.Column+cell.ColSpan]
+			x := cols[0].Offset
+			y := rows[0].Offset
+			width := 0
+			height := 0
+			for _, col := range cols {
+				width += col.Size
+			}
+			for _, row := range rows {
+				height += row.Size
+			}
+			if x <= localX && localX < x+width && y <= localY && localY < y+height {
+				switch content := cell.Content.(type) {
+				case MouseableDrawableInteractive:
+					content.MouseEvent(localX-x, localY-y, event)
+				case Mouseable:
+					content.MouseEvent(localX-x, localY-y, event)
+				case MouseHandler:
+					content.MouseEvent(localX-x, localY-y, event)
+				}
+			}
+		}
+	}
+}
+
 func (grid *Grid) reflow(ctx *Context) {
 	grid.rowLayout = nil
 	grid.columnLayout = nil
@@ -150,10 +196,10 @@ func (grid *Grid) reflow(ctx *Context) {
 		nweights := 0
 		for _, spec := range *specs {
 			if spec.Strategy == SIZE_EXACT {
-				exact += spec.Size
+				exact += spec.Size()
 			} else if spec.Strategy == SIZE_WEIGHT {
 				nweights += 1
-				weight += spec.Size
+				weight += spec.Size()
 			}
 		}
 		offset := 0
@@ -164,9 +210,9 @@ func (grid *Grid) reflow(ctx *Context) {
 		for _, spec := range *specs {
 			layout := gridLayout{Offset: offset}
 			if spec.Strategy == SIZE_EXACT {
-				layout.Size = spec.Size
+				layout.Size = spec.Size()
 			} else if spec.Strategy == SIZE_WEIGHT {
-				proportion := float64(spec.Size) / float64(weight)
+				proportion := float64(spec.Size()) / float64(weight)
 				size := proportion * float64(extent-exact)
 				if remainingExact > 0 {
 					extraExact := int(math.Ceil(proportion * float64(remainingExact)))
@@ -243,3 +289,7 @@ func (grid *Grid) cellInvalidated(drawable Drawable) {
 	cell.invalid.Store(true)
 	grid.DoInvalidate(grid)
 }
+
+func Const(i int) func() int {
+	return func() int { return i }
+}
