@@ -10,6 +10,7 @@ import (
 	"path"
 	"strconv"
 	"strings"
+	"sync"
 
 	"github.com/gdamore/tcell/v2"
 	"github.com/go-ini/ini"
@@ -46,7 +47,6 @@ type AccountWizard struct {
 	steps     []*ui.Grid
 	focus     int
 	temporary bool
-	testing   bool
 	// CONFIGURE_BASICS
 	accountName *ui.TextInput
 	email       *ui.TextInput
@@ -73,23 +73,38 @@ type AccountWizard struct {
 	complete []ui.Interactive
 }
 
+func showPasswordWarning(aerc *Aerc) {
+	title := "The Wizard will store your passwords in plaintext"
+	text := "It is recommended to remove the plaintext passwords " +
+		"and use your personal password store with " +
+		"'source-cred-cmd' and 'outgoing-cred-cmd' after the setup."
+	warning := NewSelectorDialog(
+		title, text, []string{"OK"}, 0,
+		aerc.SelectedAccountUiConfig(),
+		func(_ string, _ error) {
+			aerc.CloseDialog()
+		},
+	)
+	aerc.AddDialog(warning)
+}
+
 func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	wizard := &AccountWizard{
-		accountName:  ui.NewTextInput("", conf.Ui).Prompt("> "),
+		accountName:  ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		aerc:         aerc,
 		conf:         conf,
 		temporary:    false,
 		copySent:     true,
-		email:        ui.NewTextInput("", conf.Ui).Prompt("> "),
-		fullName:     ui.NewTextInput("", conf.Ui).Prompt("> "),
-		imapPassword: ui.NewTextInput("", conf.Ui).Prompt("] ").Password(true),
-		imapServer:   ui.NewTextInput("", conf.Ui).Prompt("> "),
+		email:        ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		fullName:     ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		imapPassword: ui.NewTextInput("", &conf.Ui).Prompt("] ").Password(true),
+		imapServer:   ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		imapStr:      ui.NewText("imaps://", conf.Ui.GetStyle(config.STYLE_DEFAULT)),
-		imapUsername: ui.NewTextInput("", conf.Ui).Prompt("> "),
-		smtpPassword: ui.NewTextInput("", conf.Ui).Prompt("] ").Password(true),
-		smtpServer:   ui.NewTextInput("", conf.Ui).Prompt("> "),
+		imapUsername: ui.NewTextInput("", &conf.Ui).Prompt("> "),
+		smtpPassword: ui.NewTextInput("", &conf.Ui).Prompt("] ").Password(true),
+		smtpServer:   ui.NewTextInput("", &conf.Ui).Prompt("> "),
 		smtpStr:      ui.NewText("smtps://", conf.Ui.GetStyle(config.STYLE_DEFAULT)),
-		smtpUsername: ui.NewTextInput("", conf.Ui).Prompt("> "),
+		smtpUsername: ui.NewTextInput("", &conf.Ui).Prompt("> "),
 	}
 
 	// Autofill some stuff for the user
@@ -120,10 +135,12 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.imapUri()
 		wizard.smtpUri()
 	})
+	var once sync.Once
 	wizard.imapPassword.OnChange(func(_ *ui.TextInput) {
 		wizard.smtpPassword.Set(wizard.imapPassword.String())
 		wizard.imapUri()
 		wizard.smtpUri()
+		once.Do(func() { showPasswordWarning(aerc) })
 	})
 	wizard.smtpServer.OnChange(func(_ *ui.TextInput) {
 		wizard.smtpUri()
@@ -179,7 +196,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		At(7, 0)
 	basics.AddChild(wizard.email).
 		At(8, 0)
-	selector := NewSelector([]string{"Next"}, 0, conf.Ui).
+	selector := NewSelector([]string{"Next"}, 0, &conf.Ui).
 		OnChoose(func(option string) {
 			email := wizard.email.String()
 			if strings.ContainsRune(email, '@') {
@@ -194,7 +211,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 					}
 					wizard.imapUri()
 				}
-				hostport, srv = getSRV(server, []string{"submission"})
+				hostport, _ = getSRV(server, []string{"submission"})
 				if hostport != "" {
 					wizard.smtpServer.Set(hostport)
 					wizard.smtpMode = SMTP_STARTTLS
@@ -265,7 +282,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"IMAP over SSL/TLS",
 		"IMAP with STARTTLS",
 		"Insecure IMAP",
-	}, 0, conf.Ui).Chooser(true).OnSelect(func(option string) {
+	}, 0, &conf.Ui).Chooser(true).OnSelect(func(option string) {
 		switch option {
 		case "IMAP over SSL/TLS":
 			wizard.imapMode = IMAP_OVER_TLS
@@ -277,7 +294,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.imapUri()
 	})
 	incoming.AddChild(imapMode).At(11, 0)
-	selector = NewSelector([]string{"Previous", "Next"}, 1, conf.Ui).
+	selector = NewSelector([]string{"Previous", "Next"}, 1, &conf.Ui).
 		OnChoose(wizard.advance)
 	incoming.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(12, 0)
 	incoming.AddChild(wizard.imapStr).At(13, 0)
@@ -347,7 +364,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"SMTP over SSL/TLS",
 		"SMTP with STARTTLS",
 		"Insecure SMTP",
-	}, 0, conf.Ui).Chooser(true).OnSelect(func(option string) {
+	}, 0, &conf.Ui).Chooser(true).OnSelect(func(option string) {
 		switch option {
 		case "SMTP over SSL/TLS":
 			wizard.smtpMode = SMTP_OVER_TLS
@@ -359,7 +376,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.smtpUri()
 	})
 	outgoing.AddChild(smtpMode).At(11, 0)
-	selector = NewSelector([]string{"Previous", "Next"}, 1, conf.Ui).
+	selector = NewSelector([]string{"Previous", "Next"}, 1, &conf.Ui).
 		OnChoose(wizard.advance)
 	outgoing.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(12, 0)
 	outgoing.AddChild(wizard.smtpStr).At(13, 0)
@@ -367,7 +384,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	outgoing.AddChild(
 		ui.NewText("Copy sent messages to 'Sent' folder?",
 			conf.Ui.GetStyle(config.STYLE_HEADER))).At(15, 0)
-	copySent := NewSelector([]string{"Yes", "No"}, 0, conf.Ui).
+	copySent := NewSelector([]string{"Yes", "No"}, 0, &conf.Ui).
 		Chooser(true).OnChoose(func(option string) {
 		switch option {
 		case "Yes":
@@ -402,7 +419,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		"Previous",
 		"Finish & open tutorial",
 		"Finish",
-	}, 1, conf.Ui).OnChoose(func(option string) {
+	}, 1, &conf.Ui).OnChoose(func(option string) {
 		switch option {
 		case "Previous":
 			wizard.advance("Previous")
@@ -493,20 +510,21 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 		return
 	}
 	sec, _ = file.NewSection(wizard.accountName.String())
-	sec.NewKey("source", wizard.imapUrl.String())
-	sec.NewKey("outgoing", wizard.smtpUrl.String())
-	sec.NewKey("default", "INBOX")
+	// these can't really fail
+	sec.NewKey("source", wizard.imapUrl.String())   //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
+	sec.NewKey("outgoing", wizard.smtpUrl.String()) //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
+	sec.NewKey("default", "INBOX")                  //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	if wizard.smtpMode == SMTP_STARTTLS {
-		sec.NewKey("smtp-starttls", "yes")
+		sec.NewKey("smtp-starttls", "yes") //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	}
-	sec.NewKey("from", fmt.Sprintf("%s <%s>",
+	sec.NewKey("from", fmt.Sprintf("%s <%s>", //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 		wizard.fullName.String(), wizard.email.String()))
 	if wizard.copySent {
-		sec.NewKey("copy-to", "Sent")
+		sec.NewKey("copy-to", "Sent") //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	}
 
 	if !wizard.temporary {
-		f, err := os.OpenFile(accountsConf, os.O_WRONLY|os.O_CREATE, 0600)
+		f, err := os.OpenFile(accountsConf, os.O_WRONLY|os.O_CREATE, 0o600)
 		if err != nil {
 			wizard.errorFor(nil, err)
 			return
@@ -522,7 +540,7 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 		Default:  "INBOX",
 		From:     sec.Key("from").String(),
 		Source:   sec.Key("source").String(),
-		Outgoing: sec.Key("outgoing").String(),
+		Outgoing: config.RemoteConfig{Value: sec.Key("outgoing").String()},
 	}
 	if wizard.smtpMode == SMTP_STARTTLS {
 		account.Params = map[string]string{
@@ -534,8 +552,7 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 	}
 	wizard.conf.Accounts = append(wizard.conf.Accounts, account)
 
-	view, err := NewAccountView(wizard.aerc, wizard.conf, &account,
-		wizard.aerc.logger, wizard.aerc, nil)
+	view, err := NewAccountView(wizard.aerc, wizard.conf, &account, wizard.aerc, nil)
 	if err != nil {
 		wizard.aerc.NewTab(errorScreen(err.Error(), wizard.conf.Ui),
 			account.Name)
@@ -690,8 +707,7 @@ func (wizard *AccountWizard) Focus(focus bool) {
 
 func (wizard *AccountWizard) Event(event tcell.Event) bool {
 	interactive := wizard.getInteractive()
-	switch event := event.(type) {
-	case *tcell.EventKey:
+	if event, ok := event.(*tcell.EventKey); ok {
 		switch event.Key() {
 		case tcell.KeyUp:
 			fallthrough
