@@ -3,13 +3,13 @@ package widgets
 import (
 	"errors"
 	"fmt"
-	"log"
 	"time"
 
 	"github.com/gdamore/tcell/v2"
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
 	"git.sr.ht/~rjarry/aerc/lib/statusline"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
@@ -29,23 +29,26 @@ type AccountView struct {
 	labels  []string
 	grid    *ui.Grid
 	host    TabHost
-	logger  *log.Logger
 	msglist *MessageList
 	worker  *types.Worker
 	state   *statusline.State
 	newConn bool // True if this is a first run after a new connection/reconnection
 	uiConf  *config.UIConfig
+
+	// Check-mail ticker
+	ticker       *time.Ticker
+	checkingMail bool
 }
 
 func (acct *AccountView) UiConfig() *config.UIConfig {
 	if dirlist := acct.Directories(); dirlist != nil {
-		return dirlist.UiConfig()
+		return dirlist.UiConfig("")
 	}
 	return acct.uiConf
 }
 
 func NewAccountView(aerc *Aerc, conf *config.AercConfig, acct *config.AccountConfig,
-	logger *log.Logger, host TabHost, deferLoop chan struct{},
+	host TabHost, deferLoop chan struct{},
 ) (*AccountView, error) {
 	acctUiConf := conf.GetUiConfig(map[config.ContextType]string{
 		config.UI_CONTEXT_ACCOUNT: acct.Name,
@@ -56,7 +59,6 @@ func NewAccountView(aerc *Aerc, conf *config.AercConfig, acct *config.AccountCon
 		aerc:   aerc,
 		conf:   conf,
 		host:   host,
-		logger: logger,
 		state:  statusline.NewState(acct.Name, len(conf.Accounts) > 1, conf.Statusline),
 		uiConf: acctUiConf,
 	}
@@ -70,20 +72,20 @@ func NewAccountView(aerc *Aerc, conf *config.AercConfig, acct *config.AccountCon
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	})
 
-	worker, err := worker.NewWorker(acct.Source, logger)
+	worker, err := worker.NewWorker(acct.Source)
 	if err != nil {
 		host.SetError(fmt.Sprintf("%s: %s", acct.Name, err))
-		logger.Printf("%s: %s\n", acct.Name, err)
+		logging.Errorf("%s: %v", acct.Name, err)
 		return view, err
 	}
 	view.worker = worker
 
-	view.dirlist = NewDirectoryList(conf, acct, logger, worker)
+	view.dirlist = NewDirectoryList(conf, acct, worker)
 	if acctUiConf.SidebarWidth > 0 {
 		view.grid.AddChild(ui.NewBordered(view.dirlist, ui.BORDER_RIGHT, acctUiConf))
 	}
 
-	view.msglist = NewMessageList(conf, logger, aerc)
+	view.msglist = NewMessageList(conf, aerc)
 	view.grid.AddChild(view.msglist).At(0, 1)
 
 	go func() {
@@ -134,7 +136,7 @@ func (acct *AccountView) UpdateStatus() {
 }
 
 func (acct *AccountView) PushStatus(status string, expiry time.Duration) {
-	acct.aerc.PushStatus(fmt.Sprintf("%s: %v", acct.acct.Name, status), expiry)
+	acct.aerc.PushStatus(fmt.Sprintf("%s: %s", acct.acct.Name, status), expiry)
 }
 
 func (acct *AccountView) PushError(err error) {
@@ -149,10 +151,6 @@ func (acct *AccountView) Worker() *types.Worker {
 	return acct.worker
 }
 
-func (acct *AccountView) Logger() *log.Logger {
-	return acct.logger
-}
-
 func (acct *AccountView) Name() string {
 	return acct.acct.Name
 }
@@ -225,8 +223,10 @@ func (acct *AccountView) SelectedMessage() (*models.MessageInfo, error) {
 }
 
 func (acct *AccountView) MarkedMessages() ([]uint32, error) {
-	store := acct.Store()
-	return store.Marked(), nil
+	if store := acct.Store(); store != nil {
+		return store.Marker().Marked(), nil
+	}
+	return nil, errors.New("no store available")
 }
 
 func (acct *AccountView) SelectedMessagePart() *PartInfo {
@@ -234,7 +234,7 @@ func (acct *AccountView) SelectedMessagePart() *PartInfo {
 }
 
 func (acct *AccountView) isSelected() bool {
-	return acct.aerc.NumTabs() > 0 && acct == acct.aerc.SelectedAccount()
+	return acct == acct.aerc.SelectedAccount()
 }
 
 func (acct *AccountView) onMessage(msg types.WorkerMessage) {
@@ -243,7 +243,7 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 		switch msg.InResponseTo().(type) {
 		case *types.Connect, *types.Reconnect:
 			acct.SetStatus(statusline.ConnectionActivity("Listing mailboxes..."))
-			acct.logger.Println("Listing mailboxes...")
+			logging.Debugf("Listing mailboxes...")
 			acct.dirlist.UpdateList(func(dirs []string) {
 				var dir string
 				for _, _dir := range dirs {
@@ -259,14 +259,14 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 					acct.dirlist.Select(dir)
 				}
 				acct.msglist.SetInitDone()
-				acct.logger.Println("Connected.")
+				logging.Infof("%s connected.", acct.acct.Name)
 				acct.SetStatus(statusline.SetConnected(true))
 				acct.newConn = true
 			})
 		case *types.Disconnect:
 			acct.dirlist.ClearList()
 			acct.msglist.SetStore(nil)
-			acct.logger.Println("Disconnected.")
+			logging.Infof("%s disconnected.", acct.acct.Name)
 			acct.SetStatus(statusline.SetConnected(false))
 		case *types.OpenDirectory:
 			if store, ok := acct.dirlist.SelectedMsgStore(); ok {
@@ -283,27 +283,29 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 		case *types.RemoveDirectory:
 			acct.dirlist.UpdateList(nil)
 		case *types.FetchMessageHeaders:
-			if acct.newConn && acct.AccountConfig().CheckMail.Minutes() > 0 {
-				acct.newConn = false
-				acct.CheckMail()
+			if acct.newConn {
+				acct.checkMailOnStartup()
 			}
 		}
 	case *types.DirectoryInfo:
 		if store, ok := acct.dirlist.MsgStore(msg.Info.Name); ok {
 			store.Update(msg)
 		} else {
+			name := msg.Info.Name
 			store = lib.NewMessageStore(acct.worker, msg.Info,
 				acct.GetSortCriteria(),
-				acct.UiConfig().ThreadingEnabled,
-				acct.UiConfig().ForceClientThreads,
+				acct.dirlist.UiConfig(name).ThreadingEnabled,
+				acct.dirlist.UiConfig(name).ForceClientThreads,
+				acct.dirlist.UiConfig(name).ClientThreadsDelay,
 				func(msg *models.MessageInfo) {
 					acct.conf.Triggers.ExecNewEmail(acct.acct,
 						acct.conf, msg)
 				}, func() {
-					if acct.UiConfig().NewMessageBell {
+					if acct.dirlist.UiConfig(name).NewMessageBell {
 						acct.host.Beep()
 					}
 				})
+			store.SetMarker(marker.New(store))
 			acct.dirlist.SetMsgStore(msg.Info.Name, store)
 		}
 	case *types.DirectoryContents:
@@ -314,6 +316,9 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 			store.Update(msg)
 			acct.SetStatus(statusline.Threading(store.ThreadedView()))
 		}
+		if acct.newConn && len(msg.Uids) == 0 {
+			acct.checkMailOnStartup()
+		}
 	case *types.DirectoryThreaded:
 		if store, ok := acct.dirlist.SelectedMsgStore(); ok {
 			if acct.msglist.Store() == nil {
@@ -322,6 +327,9 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 			store.Update(msg)
 			acct.SetStatus(statusline.Threading(store.ThreadedView()))
 		}
+		if acct.newConn && len(msg.Threads) == 0 {
+			acct.checkMailOnStartup()
+		}
 	case *types.FullMessage:
 		if store, ok := acct.dirlist.SelectedMsgStore(); ok {
 			store.Update(msg)
@@ -338,85 +346,123 @@ func (acct *AccountView) onMessage(msg types.WorkerMessage) {
 			store.Update(msg)
 		}
 	case *types.MessagesCopied:
-		// Only update the destination destStore if it is initialized
-		if destStore, ok := acct.dirlist.MsgStore(msg.Destination); ok {
-			var recent, unseen int
-			for _, uid := range msg.Uids {
-				// Get the message from the originating store
-				msg, ok := acct.Store().Messages[uid]
-				if !ok {
-					continue
-				}
-				seen := false
-				for _, flag := range msg.Flags {
-					if flag == models.SeenFlag {
-						seen = true
-					}
-					if flag == models.RecentFlag {
-						recent = recent + 1
-					}
-				}
-				if !seen {
-					unseen = unseen + 1
-				}
-			}
-			destStore.DirInfo.Recent += recent
-			destStore.DirInfo.Unseen += unseen
-			destStore.DirInfo.Exists += len(msg.Uids)
-			// True. For imap, we don't have the message in the store until we
-			// Select so we need to rely on the math we just did for accurate
-			// counts
-			destStore.DirInfo.AccurateCounts = true
-		}
+		acct.updateDirCounts(msg.Destination, msg.Uids)
+	case *types.MessagesMoved:
+		acct.updateDirCounts(msg.Destination, msg.Uids)
 	case *types.LabelList:
 		acct.labels = msg.Labels
 	case *types.ConnError:
-		acct.logger.Printf("connection error: [%s] %v", acct.acct.Name, msg.Error)
+		logging.Errorf("%s connection error: %v", acct.acct.Name, msg.Error)
 		acct.SetStatus(statusline.SetConnected(false))
 		acct.PushError(msg.Error)
 		acct.msglist.SetStore(nil)
 		acct.worker.PostAction(&types.Reconnect{}, nil)
 	case *types.Error:
-		acct.logger.Printf("%v", msg.Error)
+		logging.Errorf("%s unexpected error: %v", acct.acct.Name, msg.Error)
 		acct.PushError(msg.Error)
 	}
 	acct.UpdateStatus()
 }
 
+func (acct *AccountView) updateDirCounts(destination string, uids []uint32) {
+	// Only update the destination destStore if it is initialized
+	if destStore, ok := acct.dirlist.MsgStore(destination); ok {
+		var recent, unseen int
+		var accurate bool = true
+		for _, uid := range uids {
+			// Get the message from the originating store
+			msg, ok := acct.Store().Messages[uid]
+			if !ok {
+				continue
+			}
+			// If message that was not yet loaded is copied
+			if msg == nil {
+				accurate = false
+				break
+			}
+			seen := false
+			for _, flag := range msg.Flags {
+				if flag == models.SeenFlag {
+					seen = true
+				}
+				if flag == models.RecentFlag {
+					recent++
+				}
+			}
+			if !seen {
+				unseen++
+			}
+		}
+		if accurate {
+			destStore.DirInfo.Recent += recent
+			destStore.DirInfo.Unseen += unseen
+			destStore.DirInfo.Exists += len(uids)
+			// True. For imap, we don't have the message in the store until we
+			// Select so we need to rely on the math we just did for accurate
+			// counts
+			destStore.DirInfo.AccurateCounts = true
+		} else {
+			destStore.DirInfo.Exists += len(uids)
+			// False to trigger recount of recent/unseen
+			destStore.DirInfo.AccurateCounts = false
+		}
+	}
+}
+
 func (acct *AccountView) GetSortCriteria() []*types.SortCriterion {
 	if len(acct.UiConfig().Sort) == 0 {
 		return nil
 	}
 	criteria, err := sort.GetSortCriteria(acct.UiConfig().Sort)
 	if err != nil {
-		acct.PushError(fmt.Errorf("ui sort: %v", err))
+		acct.PushError(fmt.Errorf("ui sort: %w", err))
 		return nil
 	}
 	return criteria
 }
 
 func (acct *AccountView) CheckMail() {
+	if acct.checkingMail {
+		return
+	}
 	// Exclude selected mailbox, per IMAP specification
-	exclude := append(acct.AccountConfig().CheckMailExclude, acct.dirlist.Selected())
+	exclude := append(acct.AccountConfig().CheckMailExclude, acct.dirlist.Selected()) //nolint:gocritic // intentional append to different slice
 	dirs := acct.dirlist.List()
 	dirs = acct.dirlist.FilterDirs(dirs, acct.AccountConfig().CheckMailInclude, false)
 	dirs = acct.dirlist.FilterDirs(dirs, exclude, true)
-	acct.logger.Printf("Checking for new mail on account %s", acct.Name())
+	logging.Infof("Checking for new mail on account %s", acct.Name())
 	acct.SetStatus(statusline.ConnectionActivity("Checking for new mail..."))
 	msg := &types.CheckMail{
 		Directories: dirs,
 		Command:     acct.acct.CheckMailCmd,
 		Timeout:     acct.acct.CheckMailTimeout,
 	}
+	acct.checkingMail = true
 	acct.worker.PostAction(msg, func(_ types.WorkerMessage) {
 		acct.SetStatus(statusline.ConnectionActivity(""))
+		acct.checkingMail = false
 	})
 }
 
+// CheckMailReset resets the check-mail timer
+func (acct *AccountView) CheckMailReset() {
+	if acct.ticker != nil {
+		d := acct.AccountConfig().CheckMail
+		acct.ticker = time.NewTicker(d)
+	}
+}
+
+func (acct *AccountView) checkMailOnStartup() {
+	if acct.AccountConfig().CheckMail.Minutes() > 0 {
+		acct.newConn = false
+		acct.CheckMail()
+	}
+}
+
 func (acct *AccountView) CheckMailTimer(d time.Duration) {
-	ticker := time.NewTicker(d)
+	acct.ticker = time.NewTicker(d)
 	go func() {
-		for range ticker.C {
+		for range acct.ticker.C {
 			if !acct.state.Connected() {
 				continue
 			}
