@@ -1,17 +1,12 @@
 package widgets
 
 import (
-	"bufio"
 	"bytes"
 	"fmt"
 	"io"
-	"io/ioutil"
-	"mime"
-	"net/http"
 	"net/textproto"
 	"os"
 	"os/exec"
-	"path/filepath"
 	"strings"
 	"time"
 
@@ -23,9 +18,11 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/completer"
 	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -40,7 +37,7 @@ type Composer struct {
 	acct       *AccountView
 	aerc       *Aerc
 
-	attachments []string
+	attachments []lib.Attachment
 	editor      *Terminal
 	email       *os.File
 	grid        *ui.Grid
@@ -61,12 +58,14 @@ type Composer struct {
 	onClose []func(ti *Composer)
 
 	width int
+
+	textParts []*lib.Part
 }
 
 func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 	acctConfig *config.AccountConfig, worker *types.Worker, template string,
-	h *mail.Header, orig models.OriginalMail) (*Composer, error) {
-
+	h *mail.Header, orig models.OriginalMail,
+) (*Composer, error) {
 	if h == nil {
 		h = new(mail.Header)
 	}
@@ -78,18 +77,21 @@ func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 		}
 		if fl != nil {
 			h.SetAddressList("from", fl)
-
 		}
 	}
 
 	templateData := templates.ParseTemplateData(h, orig)
-	cmpl := completer.New(conf.Compose.AddressBookCmd, func(err error) {
+	cmd := acctConfig.AddressBookCmd
+	if cmd == "" {
+		cmd = conf.Compose.AddressBookCmd
+	}
+	cmpl := completer.New(cmd, func(err error) {
 		aerc.PushError(
 			fmt.Sprintf("could not complete header: %v", err))
-		worker.Logger.Printf("could not complete header: %v", err)
-	}, aerc.Logger())
+		logging.Errorf("could not complete header: %v", err)
+	})
 
-	email, err := ioutil.TempFile("", "aerc-compose-*.eml")
+	email, err := os.CreateTemp("", "aerc-compose-*.eml")
 	if err != nil {
 		// TODO: handle this better
 		return nil, err
@@ -116,11 +118,15 @@ func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 	c.AddSignature()
 
 	c.updateGrid()
-	c.updateCrypto()
+	err = c.updateCrypto()
+	if err != nil {
+		logging.Warnf("failed to update crypto: %v", err)
+	}
 	c.ShowTerminal()
 
 	if c.acctConfig.PgpAutoSign {
-		c.SetSign(true)
+		err = c.SetSign(true)
+		logging.Warnf("failed to enable message signing: %v", err)
 	}
 	if c.acctConfig.PgpOpportunisticEncrypt {
 		c.SetEncrypt(true)
@@ -130,7 +136,6 @@ func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 }
 
 func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
-
 	c.layout = aerc.conf.Compose.HeaderLayout
 	c.editors = make(map[string]*headerEditor)
 	c.focusable = make([]ui.MouseableDrawableInteractive, 0)
@@ -185,7 +190,61 @@ func (c *Composer) Sent() bool {
 }
 
 func (c *Composer) SetAttachKey(attach bool) error {
+	if !attach {
+		name := c.crypto.signKey + ".asc"
+		found := false
+		for _, a := range c.attachments {
+			if a.Name() == name {
+				found = true
+			}
+		}
+		if found {
+			err := c.DeleteAttachment(name)
+			if err != nil {
+				return fmt.Errorf("failed to delete attachment '%s: %w", name, err)
+			}
+		} else {
+			attach = !attach
+		}
+	}
+	if attach {
+		var s string
+		var err error
+		if c.crypto.signKey == "" {
+			if c.acctConfig.PgpKeyId != "" {
+				s = c.acctConfig.PgpKeyId
+			} else {
+				s, err = getSenderEmail(c)
+				if err != nil {
+					return err
+				}
+			}
+			c.crypto.signKey, err = c.aerc.Crypto.GetSignerKeyId(s)
+			if err != nil {
+				return err
+			}
+		}
+
+		r, err := c.aerc.Crypto.ExportKey(c.crypto.signKey)
+		if err != nil {
+			return err
+		}
+
+		c.attachments = append(c.attachments,
+			lib.NewPartAttachment(
+				lib.NewPart(
+					"application/pgp-keys",
+					map[string]string{"charset": "UTF-8"},
+					r,
+				),
+				c.crypto.signKey+".asc",
+			),
+		)
+
+	}
+
 	c.attachKey = attach
+
 	c.resetReview()
 	return nil
 }
@@ -199,7 +258,7 @@ func (c *Composer) SetSign(sign bool) error {
 	err := c.updateCrypto()
 	if err != nil {
 		c.sign = !sign
-		return fmt.Errorf("Cannot sign message: %v", err)
+		return fmt.Errorf("Cannot sign message: %w", err)
 	}
 	return nil
 }
@@ -211,7 +270,10 @@ func (c *Composer) Sign() bool {
 func (c *Composer) SetEncrypt(encrypt bool) *Composer {
 	if !encrypt {
 		c.encrypt = encrypt
-		c.updateCrypto()
+		err := c.updateCrypto()
+		if err != nil {
+			logging.Warnf("failed to update crypto: %v", err)
+		}
 		return c
 	}
 	// Check on any attempt to encrypt, and any lost focus of "to", "cc", or
@@ -234,7 +296,7 @@ func (c *Composer) Encrypt() bool {
 func (c *Composer) updateCrypto() error {
 	if c.crypto == nil {
 		uiConfig := c.acct.UiConfig()
-		c.crypto = newCryptoStatus(&uiConfig)
+		c.crypto = newCryptoStatus(uiConfig)
 	}
 	var err error
 	// Check if signKey is empty so we only run this once
@@ -284,17 +346,47 @@ func (c *Composer) updateCrypto() error {
 // Note: this does not reload the editor. You must call this before the first
 // Draw() call.
 func (c *Composer) SetContents(reader io.Reader) *Composer {
-	c.email.Seek(0, io.SeekStart)
-	io.Copy(c.email, reader)
-	c.email.Sync()
-	c.email.Seek(0, io.SeekStart)
+	_, err := c.email.Seek(0, io.SeekStart)
+	if err != nil {
+		logging.Warnf("failed to seek beginning of mail: %v", err)
+	}
+	_, err = io.Copy(c.email, reader)
+	if err != nil {
+		logging.Warnf("failed to copy mail: %v", err)
+	}
+	err = c.email.Sync()
+	if err != nil {
+		logging.Warnf("failed to sync mail: %v", err)
+	}
+	_, err = c.email.Seek(0, io.SeekStart)
+	if err != nil {
+		logging.Warnf("failed to seek beginning of mail after sync: %v", err)
+	}
 	return c
 }
 
 func (c *Composer) AppendContents(reader io.Reader) {
-	c.email.Seek(0, io.SeekEnd)
-	io.Copy(c.email, reader)
-	c.email.Sync()
+	_, err := c.email.Seek(0, io.SeekEnd)
+	if err != nil {
+		logging.Warnf("failed to seek beginning of mail: %v", err)
+	}
+	_, err = io.Copy(c.email, reader)
+	if err != nil {
+		logging.Warnf("failed to copy mail: %v", err)
+	}
+	err = c.email.Sync()
+	if err != nil {
+		logging.Warnf("failed to sync mail: %v", err)
+	}
+}
+
+func (c *Composer) AppendPart(mimetype string, params map[string]string, body io.Reader) error {
+	if !strings.HasPrefix(mimetype, "text") {
+		return fmt.Errorf("can only append text mimetypes")
+	}
+	c.textParts = append(c.textParts, lib.NewPart(mimetype, params, body))
+	c.resetReview()
+	return nil
 }
 
 func (c *Composer) AddTemplate(template string, data interface{}) error {
@@ -310,7 +402,7 @@ func (c *Composer) AddTemplate(template string, data interface{}) error {
 
 	mr, err := mail.CreateReader(templateText)
 	if err != nil {
-		return fmt.Errorf("Template loading failed: %v", err)
+		return fmt.Errorf("Template loading failed: %w", err)
 	}
 
 	// copy the headers contained in the template to the compose headers
@@ -321,7 +413,7 @@ func (c *Composer) AddTemplate(template string, data interface{}) error {
 
 	part, err := mr.NextPart()
 	if err != nil {
-		return fmt.Errorf("Could not get body of template: %v", err)
+		return fmt.Errorf("Could not get body of template: %w", err)
 	}
 
 	c.AppendContents(part.Body)
@@ -361,7 +453,7 @@ func (c *Composer) readSignatureFromFile() []byte {
 	if err != nil {
 		return nil
 	}
-	signature, err := ioutil.ReadFile(sigFile)
+	signature, err := os.ReadFile(sigFile)
 	if err != nil {
 		c.aerc.PushError(
 			fmt.Sprintf(" Error loading signature from file: %v", sigFile))
@@ -434,11 +526,12 @@ func (c *Composer) Close() {
 }
 
 func (c *Composer) Bindings() string {
-	if c.editor == nil {
+	switch c.editor {
+	case nil:
 		return "compose::review"
-	} else if c.editor == c.focusable[c.focused] {
+	case c.focusable[c.focused]:
 		return "compose::editor"
-	} else {
+	default:
 		return "compose"
 	}
 }
@@ -476,7 +569,7 @@ func (c *Composer) Worker() *types.Worker {
 	return c.worker
 }
 
-//PrepareHeader finalizes the header, adding the value from the editors
+// PrepareHeader finalizes the header, adding the value from the editors
 func (c *Composer) PrepareHeader() (*mail.Header, error) {
 	for _, editor := range c.editors {
 		editor.storeValue()
@@ -530,7 +623,7 @@ func getRecipientsEmail(c *Composer) ([]string, error) {
 
 	// return email addresses as string slice
 	results := []string{}
-	for email, _ := range rcpts {
+	for email := range rcpts {
 		results = append(results, email)
 	}
 	return results, nil
@@ -582,8 +675,14 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 		if err != nil {
 			return err
 		}
-		cleartext.Close()
-		io.Copy(writer, &buf)
+		err = cleartext.Close()
+		if err != nil {
+			return err
+		}
+		_, err = io.Copy(writer, &buf)
+		if err != nil {
+			return fmt.Errorf("failed to write message: %w", err)
+		}
 		return nil
 
 	} else {
@@ -592,29 +691,30 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 }
 
 func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
-	if len(c.attachments) == 0 && !c.attachKey {
-		// no attachements
+	if len(c.attachments) == 0 && len(c.textParts) == 0 {
+		// no attachments
 		return writeInlineBody(header, c.email, writer)
 	} else {
-		// with attachements
+		// with attachments
 		w, err := mail.CreateWriter(writer, *header)
 		if err != nil {
 			return errors.Wrap(err, "CreateWriter")
 		}
-		if err := writeMultipartBody(c.email, w); err != nil {
+		parts := []*lib.Part{
+			lib.NewPart(
+				"text/plain",
+				map[string]string{"Charset": "UTF-8"},
+				c.email,
+			),
+		}
+		if err := writeMultipartBody(append(parts, c.textParts...), w); err != nil {
 			return errors.Wrap(err, "writeMultipartBody")
 		}
 		for _, a := range c.attachments {
-			if err := writeAttachment(a, w); err != nil {
+			if err := a.WriteTo(w); err != nil {
 				return errors.Wrap(err, "writeAttachment")
 			}
 		}
-		if c.attachKey {
-			err := c.writeKeyAttachment(w)
-			if err != nil {
-				return err
-			}
-		}
 		w.Close()
 	}
 	return nil
@@ -634,94 +734,52 @@ func writeInlineBody(header *mail.Header, body io.Reader, writer io.Writer) erro
 }
 
 // write the message body to the multipart message
-func writeMultipartBody(body io.Reader, w *mail.Writer) error {
-	bh := mail.InlineHeader{}
-	bh.SetContentType("text/plain", map[string]string{"charset": "UTF-8"})
-
+func writeMultipartBody(parts []*lib.Part, w *mail.Writer) error {
 	bi, err := w.CreateInline()
 	if err != nil {
 		return errors.Wrap(err, "CreateInline")
 	}
 	defer bi.Close()
 
-	bw, err := bi.CreatePart(bh)
-	if err != nil {
-		return errors.Wrap(err, "CreatePart")
-	}
-	defer bw.Close()
-	if _, err := io.Copy(bw, body); err != nil {
-		return errors.Wrap(err, "io.Copy")
-	}
-	return nil
-}
-
-// write the attachment specified by path to the message
-func writeAttachment(path string, writer *mail.Writer) error {
-	f, err := os.Open(path)
-	if err != nil {
-		return errors.Wrap(err, "os.Open")
-	}
-	defer f.Close()
-
-	reader := bufio.NewReader(f)
-
-	// if we have an extension, prefer that instead of trying to sniff the header.
-	// That's generally more accurate than sniffing as lots of things are zip files
-	// under the hood, e.g. most office file types
-	ext := filepath.Ext(path)
-	var mimeString string
-	if mimeString = mime.TypeByExtension(ext); mimeString != "" {
-		// found it in the DB
-	} else {
-		// Sniff the mime type instead
-		// http.DetectContentType only cares about the first 512 bytes
-		head, err := reader.Peek(512)
-		if err != nil && err != io.EOF {
-			return errors.Wrap(err, "Peek")
+	for _, part := range parts {
+		bh := mail.InlineHeader{}
+		bh.SetContentType(part.MimeType, part.Params)
+		bw, err := bi.CreatePart(bh)
+		if err != nil {
+			return errors.Wrap(err, "CreatePart")
+		}
+		defer bw.Close()
+		if _, err := io.Copy(bw, part.Body); err != nil {
+			return errors.Wrap(err, "io.Copy")
 		}
-		mimeString = http.DetectContentType(head)
-	}
-
-	// mimeString can contain type and params (like text encoding),
-	// so we need to break them apart before passing them to the headers
-	mimeType, params, err := mime.ParseMediaType(mimeString)
-	if err != nil {
-		return errors.Wrap(err, "ParseMediaType")
-	}
-	filename := filepath.Base(path)
-	params["name"] = filename
-
-	// set header fields
-	ah := mail.AttachmentHeader{}
-	ah.SetContentType(mimeType, params)
-	// setting the filename auto sets the content disposition
-	ah.SetFilename(filename)
-
-	aw, err := writer.CreateAttachment(ah)
-	if err != nil {
-		return errors.Wrap(err, "CreateAttachment")
-	}
-	defer aw.Close()
-
-	if _, err := reader.WriteTo(aw); err != nil {
-		return errors.Wrap(err, "reader.WriteTo")
 	}
 
 	return nil
 }
 
 func (c *Composer) GetAttachments() []string {
-	return c.attachments
+	var names []string
+	for _, a := range c.attachments {
+		names = append(names, a.Name())
+	}
+	return names
 }
 
 func (c *Composer) AddAttachment(path string) {
-	c.attachments = append(c.attachments, path)
+	c.attachments = append(c.attachments, lib.NewFileAttachment(path))
 	c.resetReview()
 }
 
-func (c *Composer) DeleteAttachment(path string) error {
+func (c *Composer) AddPartAttachment(name string, mimetype string, params map[string]string, body io.Reader) {
+	c.attachments = append(c.attachments, lib.NewPartAttachment(
+		lib.NewPart(mimetype, params, body), name,
+	))
+	c.resetReview()
+}
+
+func (c *Composer) DeleteAttachment(name string) error {
 	for i, a := range c.attachments {
-		if a == path {
+		if a.Name() == name {
 			c.attachments = append(c.attachments[:i], c.attachments[i+1:]...)
 			c.resetReview()
 			return nil
@@ -740,10 +798,8 @@ func (c *Composer) resetReview() {
 }
 
 func (c *Composer) termEvent(event tcell.Event) bool {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
-		switch event.Buttons() {
-		case tcell.Button1:
+	if event, ok := event.(*tcell.EventMouse); ok {
+		if event.Buttons() == tcell.Button1 {
 			c.FocusTerminal()
 			return true
 		}
@@ -903,11 +959,12 @@ type headerEditor struct {
 	header   *mail.Header
 	focused  bool
 	input    *ui.TextInput
-	uiConfig config.UIConfig
+	uiConfig *config.UIConfig
 }
 
 func newHeaderEditor(name string, h *mail.Header,
-	uiConfig config.UIConfig) *headerEditor {
+	uiConfig *config.UIConfig,
+) *headerEditor {
 	he := &headerEditor{
 		input:    ui.NewTextInput("", uiConfig),
 		name:     name,
@@ -918,8 +975,8 @@ func newHeaderEditor(name string, h *mail.Header,
 	return he
 }
 
-//extractHumanHeaderValue extracts the human readable string for key from the
-//header. If a parsing error occurs the raw value is returned
+// extractHumanHeaderValue extracts the human readable string for key from the
+// header. If a parsing error occurs the raw value is returned
 func extractHumanHeaderValue(key string, h *mail.Header) string {
 	var val string
 	var err error
@@ -938,20 +995,25 @@ func extractHumanHeaderValue(key string, h *mail.Header) string {
 	return val
 }
 
-//loadValue loads the value of he.name form the underlying header
-//the value is decoded and meant for human consumption.
-//decoding issues are ignored and return their raw values
+// loadValue loads the value of he.name form the underlying header
+// the value is decoded and meant for human consumption.
+// decoding issues are ignored and return their raw values
 func (he *headerEditor) loadValue() {
 	he.input.Set(extractHumanHeaderValue(he.name, he.header))
 	he.input.Invalidate()
 }
 
-//storeValue writes the current state back to the underlying header.
-//errors are ignored
+// storeValue writes the current state back to the underlying header.
+// errors are ignored
 func (he *headerEditor) storeValue() {
 	val := he.input.String()
 	switch strings.ToLower(he.name) {
 	case "to", "from", "cc", "bcc":
+		if strings.TrimSpace(val) == "" {
+			// if header is empty, delete it
+			he.header.Del(he.name)
+			return
+		}
 		list, err := mail.ParseAddressList(val)
 		if err == nil {
 			he.header.SetAddressList(he.name, list)
@@ -965,13 +1027,6 @@ func (he *headerEditor) storeValue() {
 	}
 }
 
-//setValue overwrites the current value of the header editor and flushes it
-//to the underlying header
-func (he *headerEditor) setValue(val string) {
-	he.input.Set(val)
-	he.storeValue()
-}
-
 func (he *headerEditor) Draw(ctx *ui.Context) {
 	name := textproto.CanonicalMIMEHeaderKey(he.name)
 	// Extra character to put a blank cell between the header and the input
@@ -984,10 +1039,8 @@ func (he *headerEditor) Draw(ctx *ui.Context) {
 }
 
 func (he *headerEditor) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
-		switch event.Buttons() {
-		case tcell.Button1:
+	if event, ok := event.(*tcell.EventMouse); ok {
+		if event.Buttons() == tcell.Button1 {
 			he.focused = true
 		}
 
@@ -1063,7 +1116,7 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 			inputs = append(inputs, config.FormatKeyStrokes(input))
 		}
 		actions = append(actions, fmt.Sprintf("  %-6s  %-40s  %s",
-			strings.Join(inputs[:], ", "), name, cmd))
+			strings.Join(inputs, ", "), name, cmd))
 	}
 
 	spec := []ui.GridSpec{
@@ -1077,8 +1130,12 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 	for i := 0; i < len(composer.attachments)-1; i++ {
 		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
 	}
-	if composer.attachKey {
+	if len(composer.textParts) > 0 {
 		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		for i := 0; i < len(composer.textParts); i++ {
+			spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_EXACT, Size: ui.Const(1)})
+		}
 	}
 	// make the last element fill remaining space
 	spec = append(spec, ui.GridSpec{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)})
@@ -1105,21 +1162,29 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 		grid.AddChild(ui.NewText("Attachments:",
 			uiConfig.GetStyle(config.STYLE_TITLE))).At(i, 0)
 		i += 1
-		if composer.attachKey {
-			grid.AddChild(ui.NewText(composer.crypto.signKey+".asc",
-				uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
-			i += 1
-		}
-		if len(composer.attachments) == 0 && !composer.attachKey {
+		if len(composer.attachments) == 0 {
 			grid.AddChild(ui.NewText("(none)",
 				uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+			i += 1
 		} else {
 			for _, a := range composer.attachments {
-				grid.AddChild(ui.NewText(a, uiConfig.GetStyle(config.STYLE_DEFAULT))).
+				grid.AddChild(ui.NewText(a.Name(), uiConfig.GetStyle(config.STYLE_DEFAULT))).
 					At(i, 0)
 				i += 1
 			}
 		}
+		if len(composer.textParts) > 0 {
+			grid.AddChild(ui.NewText("Parts:",
+				uiConfig.GetStyle(config.STYLE_TITLE))).At(i, 0)
+			i += 1
+			grid.AddChild(ui.NewText("text/plain", uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+			i += 1
+			for _, p := range composer.textParts {
+				grid.AddChild(ui.NewText(p.MimeType, uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+				i += 1
+			}
+
+		}
 	}
 
 	return &reviewMessage{
@@ -1207,57 +1272,9 @@ func (c *Composer) checkEncryptionKeys(_ string) bool {
 	// If callbacks were registered, encrypt will be set when user removes
 	// recipients with missing keys
 	c.encrypt = true
-	c.updateCrypto()
+	err = c.updateCrypto()
+	if err != nil {
+		logging.Warnf("failed update crypto: %v", err)
+	}
 	return true
 }
-
-func (c *Composer) writeKeyAttachment(w *mail.Writer) error {
-	// Verify key exists and get keyid
-	cp := c.aerc.Crypto
-	var (
-		err error
-		s   string
-	)
-	if c.crypto.signKey == "" {
-		if c.acctConfig.PgpKeyId != "" {
-			s = c.acctConfig.PgpKeyId
-		} else {
-			s, err = getSenderEmail(c)
-			if err != nil {
-				return err
-			}
-		}
-		c.crypto.signKey, err = cp.GetSignerKeyId(s)
-		if err != nil {
-			return err
-		}
-	}
-	// Get the key in armor format
-	r, err := cp.ExportKey(c.crypto.signKey)
-	if err != nil {
-		c.aerc.PushError(err.Error())
-		return err
-	}
-	filename := c.crypto.signKey + ".asc"
-	mimeType := "application/pgp-keys"
-	params := map[string]string{
-		"charset": "UTF-8",
-		"name":    filename,
-	}
-	// set header fields
-	ah := mail.AttachmentHeader{}
-	ah.SetContentType(mimeType, params)
-	// setting the filename auto sets the content disposition
-	ah.SetFilename(filename)
-
-	aw, err := w.CreateAttachment(ah)
-	if err != nil {
-		return errors.Wrap(err, "CreateKeyAttachment")
-	}
-	defer aw.Close()
-
-	if _, err := io.Copy(aw, r); err != nil {
-		return errors.Wrap(err, "io.Copy")
-	}
-	return nil
-}
