@@ -2,7 +2,6 @@ package widgets
 
 import (
 	"fmt"
-	"log"
 	"math"
 	"strings"
 
@@ -14,6 +13,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -22,7 +22,6 @@ type MessageList struct {
 	ui.Invalidatable
 	Scrollable
 	conf          *config.AercConfig
-	logger        *log.Logger
 	height        int
 	nmsgs         int
 	spinner       *Spinner
@@ -31,10 +30,9 @@ type MessageList struct {
 	aerc          *Aerc
 }
 
-func NewMessageList(conf *config.AercConfig, logger *log.Logger, aerc *Aerc) *MessageList {
+func NewMessageList(conf *config.AercConfig, aerc *Aerc) *MessageList {
 	ml := &MessageList{
 		conf:          conf,
-		logger:        logger,
 		spinner:       NewSpinner(&conf.Ui),
 		isInitalizing: true,
 		aerc:          aerc,
@@ -72,7 +70,11 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 
 	ml.UpdateScroller(ml.height, len(store.Uids()))
 	if store := ml.Store(); store != nil && len(store.Uids()) > 0 {
-		ml.EnsureScroll(store.SelectedIndex())
+		idx := store.FindIndexByUid(store.SelectedUid())
+		if idx < 0 {
+			idx = len(store.Uids()) - 1
+		}
+		ml.EnsureScroll(len(store.Uids()) - idx - 1)
 	}
 
 	textWidth := ctx.Width()
@@ -88,13 +90,13 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 		row          int = 0
 	)
 
-	if uiConfig.ThreadingEnabled || store.BuildThreads() {
-		threads := store.Threads
+	if store.ThreadedView() {
+		threads := store.Threads()
 		counter := len(store.Uids())
 
 		for i := len(threads) - 1; i >= 0; i-- {
 			var lastSubject string
-			threads[i].Walk(func(t *types.Thread, _ int, currentErr error) error {
+			err := threads[i].Walk(func(t *types.Thread, _ int, currentErr error) error {
 				if currentErr != nil {
 					return currentErr
 				}
@@ -103,7 +105,7 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 				}
 				counter--
 				if counter > len(store.Uids())-1-ml.Scroll() {
-					//skip messages which are higher than the viewport
+					// skip messages which are higher than the viewport
 					return nil
 				}
 				msg := store.Messages[t.Uid]
@@ -122,7 +124,7 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 					AccountName:       acct.Name(),
 					MsgInfo:           msg,
 					MsgNum:            row,
-					MsgIsMarked:       store.IsMarked(t.Uid),
+					MsgIsMarked:       store.Marker().IsMarked(t.Uid),
 					ThreadPrefix:      prefix,
 					ThreadSameSubject: normalizedSubject == lastSubject,
 				}
@@ -133,6 +135,9 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 				row++
 				return nil
 			})
+			if err != nil {
+				logging.Warnf("failed to walk threads: %v", err)
+			}
 			if row >= ctx.Height() {
 				break
 			}
@@ -147,7 +152,7 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 				AccountName: acct.Name(),
 				MsgInfo:     msg,
 				MsgNum:      row,
-				MsgIsMarked: store.IsMarked(uid),
+				MsgIsMarked: store.Marker().IsMarked(uid),
 			}
 			if ml.drawRow(textWidth, ctx, uid, row, &needsHeaders, fmtCtx) {
 				break
@@ -194,14 +199,23 @@ func (ml *MessageList) drawRow(textWidth int, ctx *ui.Context, uid uint32, row i
 		return false
 	}
 
-	confParams := map[config.ContextType]string{
-		config.UI_CONTEXT_ACCOUNT: acct.AccountConfig().Name,
-		config.UI_CONTEXT_FOLDER:  acct.Directories().Selected(),
+	// TODO deprecate subject contextual UIs? Only related setting is styleset,
+	// should implement a better per-message styling method
+	// Check if we have any applicable ContextualUIConfigs
+	confs := ml.aerc.conf.GetContextualUIConfigs()
+	uiConfig := acct.Directories().UiConfig(store.DirInfo.Name)
+	for _, c := range confs {
+		if c.ContextType == config.UI_CONTEXT_SUBJECT && msg.Envelope != nil {
+			if c.Regex.Match([]byte(msg.Envelope.Subject)) {
+				confParams := map[config.ContextType]string{
+					config.UI_CONTEXT_ACCOUNT: acct.AccountConfig().Name,
+					config.UI_CONTEXT_FOLDER:  acct.Directories().Selected(),
+					config.UI_CONTEXT_SUBJECT: msg.Envelope.Subject,
+				}
+				uiConfig = ml.conf.GetUiConfig(confParams)
+			}
+		}
 	}
-	if msg.Envelope != nil {
-		confParams[config.UI_CONTEXT_SUBJECT] = msg.Envelope.Subject
-	}
-	uiConfig := ml.conf.GetUiConfig(confParams)
 
 	msg_styles := []config.StyleObject{}
 	// unread message
@@ -232,13 +246,13 @@ func (ml *MessageList) drawRow(textWidth int, ctx *ui.Context, uid uint32, row i
 	}
 
 	// marked message
-	if store.IsMarked(msg.Uid) {
+	if store.Marker().IsMarked(msg.Uid) {
 		msg_styles = append(msg_styles, config.STYLE_MSGLIST_MARKED)
 	}
 
 	var style tcell.Style
 	// current row
-	if row == ml.store.SelectedIndex()-ml.Scroll() {
+	if msg.Uid == ml.store.SelectedUid() {
 		style = uiConfig.GetComposedStyleSelected(config.STYLE_MSGLIST_DEFAULT, msg_styles)
 	} else {
 		style = uiConfig.GetComposedStyle(config.STYLE_MSGLIST_DEFAULT, msg_styles)
@@ -276,8 +290,7 @@ func (ml *MessageList) drawScrollbar(ctx *ui.Context) {
 }
 
 func (ml *MessageList) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
+	if event, ok := event.(*tcell.EventMouse); ok {
 		switch event.Buttons() {
 		case tcell.Button1:
 			if ml.aerc == nil {
@@ -336,26 +349,6 @@ func (ml *MessageList) storeUpdate(store *lib.MessageStore) {
 	if ml.Store() != store {
 		return
 	}
-	uids := store.Uids()
-
-	if len(uids) > 0 {
-		// When new messages come in, advance the cursor accordingly
-		// Note that this assumes new messages are appended to the top, which
-		// isn't necessarily true once we implement SORT... ideally we'd look
-		// for the previously selected UID.
-		if len(uids) > ml.nmsgs && ml.nmsgs != 0 {
-			for i := 0; i < len(uids)-ml.nmsgs; i++ {
-				ml.Store().Next()
-			}
-		}
-		if len(uids) < ml.nmsgs && ml.nmsgs != 0 {
-			for i := 0; i < ml.nmsgs-len(uids); i++ {
-				ml.Store().Prev()
-			}
-		}
-		ml.nmsgs = len(uids)
-	}
-
 	ml.Invalidate()
 }
 
@@ -366,7 +359,8 @@ func (ml *MessageList) SetStore(store *lib.MessageStore) {
 	ml.store = store
 	if store != nil {
 		ml.spinner.Stop()
-		ml.nmsgs = len(store.Uids())
+		uids := store.Uids()
+		ml.nmsgs = len(uids)
 		store.OnUpdate(ml.storeUpdate)
 		store.OnFilterChange(func(store *lib.MessageStore) {
 			if ml.Store() != store {
@@ -394,14 +388,25 @@ func (ml *MessageList) Empty() bool {
 }
 
 func (ml *MessageList) Selected() *models.MessageInfo {
-	store := ml.Store()
-	uids := store.Uids()
-	return store.Messages[uids[len(uids)-ml.store.SelectedIndex()-1]]
+	return ml.Store().Selected()
 }
 
 func (ml *MessageList) Select(index int) {
+	// Note that the msgstore.Select function expects a uid as argument
+	// whereas the msglist.Select expects the message number
 	store := ml.Store()
-	store.Select(index)
+	uids := store.Uids()
+	if len(uids) == 0 {
+		return
+	}
+	uidIdx := len(uids) - index - 1
+	if uidIdx >= len(store.Uids()) {
+		uidIdx = 0
+	} else if uidIdx < 0 {
+		uidIdx = len(store.Uids()) - 1
+	}
+	store.Select(store.Uids()[uidIdx])
+
 	ml.Invalidate()
 }
 
