@@ -52,8 +52,8 @@ type PartSwitcher struct {
 }
 
 func NewMessageViewer(acct *AccountView,
-	conf *config.AercConfig, msg lib.MessageView) *MessageViewer {
-
+	conf *config.AercConfig, msg lib.MessageView,
+) *MessageViewer {
 	hf := HeaderLayoutFilter{
 		layout: HeaderLayout(conf.Viewer.HeaderLayout),
 		keep: func(msg *models.MessageInfo, header string) bool {
@@ -178,11 +178,11 @@ func fmtHeader(msg *models.MessageInfo, header string, timefmt string) string {
 
 func enumerateParts(acct *AccountView, conf *config.AercConfig,
 	msg lib.MessageView, body *models.BodyStructure,
-	index []int) ([]*PartViewer, error) {
-
+	index []int,
+) ([]*PartViewer, error) {
 	var parts []*PartViewer
 	for i, part := range body.Parts {
-		curindex := append(index, i+1)
+		curindex := append(index, i+1) //nolint:gocritic // intentional append to different slice
 		if part.MIMEType == "multipart" {
 			// Multipart meta-parts are faked
 			pv := &PartViewer{part: part}
@@ -205,8 +205,8 @@ func enumerateParts(acct *AccountView, conf *config.AercConfig,
 }
 
 func createSwitcher(acct *AccountView, switcher *PartSwitcher,
-	conf *config.AercConfig, msg lib.MessageView) error {
-
+	conf *config.AercConfig, msg lib.MessageView,
+) error {
 	var err error
 	switcher.selected = -1
 	switcher.showHeaders = conf.Viewer.ShowHeaders
@@ -229,7 +229,7 @@ func createSwitcher(acct *AccountView, switcher *PartSwitcher,
 			return err
 		}
 		selectedPriority := -1
-		fmt.Printf("Selecting best message from %v\n", conf.Viewer.Alternatives)
+		logging.Infof("Selecting best message from %v", conf.Viewer.Alternatives)
 		for i, pv := range switcher.parts {
 			pv.OnInvalidate(func(_ ui.Drawable) {
 				switcher.Invalidate()
@@ -302,17 +302,16 @@ func (mv *MessageViewer) SelectedMessage() (*models.MessageInfo, error) {
 }
 
 func (mv *MessageViewer) MarkedMessages() ([]uint32, error) {
-	store := mv.Store()
-	return store.Marked(), nil
+	return mv.acct.MarkedMessages()
 }
 
 func (mv *MessageViewer) ToggleHeaders() {
 	switcher := mv.switcher
+	switcher.Cleanup()
 	mv.conf.Viewer.ShowHeaders = !mv.conf.Viewer.ShowHeaders
 	err := createSwitcher(mv.acct, switcher, mv.conf, mv.msg)
 	if err != nil {
-		mv.acct.Logger().Printf(
-			"warning: error during create switcher - %v", err)
+		logging.Errorf("cannot create switcher: %v", err)
 	}
 	switcher.Invalidate()
 }
@@ -392,6 +391,22 @@ func (mv *MessageViewer) Close() error {
 	return nil
 }
 
+func (mv *MessageViewer) UpdateScreen() {
+	if mv.switcher == nil {
+		return
+	}
+	parts := mv.switcher.parts
+	selected := mv.switcher.selected
+	if selected < 0 {
+		return
+	}
+	if len(parts) > 0 && selected < len(parts) {
+		if part := parts[selected]; part != nil {
+			part.UpdateScreen()
+		}
+	}
+}
+
 func (ps *PartSwitcher) Invalidate() {
 	ps.DoInvalidate(ps)
 }
@@ -437,8 +452,7 @@ func (ps *PartSwitcher) Draw(ctx *ui.Context) {
 }
 
 func (ps *PartSwitcher) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
+	if event, ok := event.(*tcell.EventMouse); ok {
 		switch event.Buttons() {
 		case tcell.Button1:
 			height := len(ps.parts)
@@ -530,8 +544,8 @@ type PartViewer struct {
 
 func NewPartViewer(acct *AccountView, conf *config.AercConfig,
 	msg lib.MessageView, part *models.BodyStructure,
-	index []int) (*PartViewer, error) {
-
+	index []int,
+) (*PartViewer, error) {
 	var (
 		filter  *exec.Cmd
 		pager   *exec.Cmd
@@ -629,12 +643,18 @@ func (pv *PartViewer) SetSource(reader io.Reader) {
 	pv.attemptCopy()
 }
 
+func (pv *PartViewer) UpdateScreen() {
+	if pv.term != nil {
+		pv.term.Invalidate()
+	}
+}
+
 func (pv *PartViewer) attemptCopy() {
 	if pv.source == nil || pv.pager == nil || pv.pager.Process == nil {
 		return
 	}
 	if pv.filter != nil {
-		pv.copyFilterOutToPager() //delayed until we write to the sink
+		pv.copyFilterOutToPager() // delayed until we write to the sink
 	}
 	go func() {
 		defer logging.PanicHandler()
@@ -645,7 +665,10 @@ func (pv *PartViewer) attemptCopy() {
 			pv.copySourceToSinkStripAnsi()
 		} else {
 			// if it's binary we have to rely on the filter to be sane
-			io.Copy(pv.sink, pv.source)
+			_, err := io.Copy(pv.sink, pv.source)
+			if err != nil {
+				logging.Warnf("failed to copy: %w", err)
+			}
 		}
 		pv.sink.Close()
 	}()
@@ -667,14 +690,23 @@ func (pv *PartViewer) writeMailHeaders() {
 			}
 			field := fmt.Sprintf(
 				"%s: %s\n", fields.Key(), value)
-			pv.pagerin.Write([]byte(field))
+			_, err = pv.pagerin.Write([]byte(field))
+			if err != nil {
+				logging.Errorf("failed to write to stdin of pager: %v", err)
+			}
 		}
 		// virtual header
 		if len(info.Labels) != 0 {
 			labels := fmtHeader(info, "Labels", "")
-			pv.pagerin.Write([]byte(fmt.Sprintf("Labels: %s\n", labels)))
+			_, err := pv.pagerin.Write([]byte(fmt.Sprintf("Labels: %s\n", labels)))
+			if err != nil {
+				logging.Errorf("failed to write to stdin of pager: %v", err)
+			}
+		}
+		_, err := pv.pagerin.Write([]byte{'\n'})
+		if err != nil {
+			logging.Errorf("failed to write to stdin of pager: %v", err)
 		}
-		pv.pagerin.Write([]byte{'\n'})
 	}
 }
 
@@ -689,7 +721,10 @@ func (pv *PartViewer) hyperlinks(r io.Reader) (reader io.Reader) {
 func (pv *PartViewer) copyFilterOutToPager() {
 	stdout, _ := pv.filter.StdoutPipe()
 	stderr, _ := pv.filter.StderrPipe()
-	pv.filter.Start()
+	err := pv.filter.Start()
+	if err != nil {
+		logging.Warnf("failed to start filter: %v", err)
+	}
 	ch := make(chan interface{})
 	go func() {
 		defer logging.PanicHandler()
@@ -718,7 +753,10 @@ func (pv *PartViewer) copyFilterOutToPager() {
 
 		<-ch
 		<-ch
-		pv.filter.Wait()
+		err := pv.filter.Wait()
+		if err != nil {
+			logging.Warnf("failed to wait for the filter process: %v", err)
+		}
 		pv.pagerin.Close()
 	}()
 }
@@ -732,7 +770,10 @@ func (pv *PartViewer) copySourceToSinkStripAnsi() {
 	for scanner.Scan() {
 		text := scanner.Text()
 		text = ansi.ReplaceAllString(text, "")
-		io.WriteString(pv.sink, text+"\n")
+		_, err := io.WriteString(pv.sink, text+"\n")
+		if err != nil {
+			logging.Warnf("failed write ", err)
+		}
 	}
 	if err := scanner.Err(); err != nil {
 		fmt.Fprintf(os.Stderr, "failed to read line: %v\n", err)
@@ -764,7 +805,7 @@ func newNoFilterConfigured(pv *PartViewer) *ui.Grid {
 			inputs = append(inputs, config.FormatKeyStrokes(input))
 		}
 		actions = append(actions, fmt.Sprintf("  %-6s  %-29s  %s",
-			strings.Join(inputs[:], ", "), name, cmd))
+			strings.Join(inputs, ", "), name, cmd))
 	}
 
 	spec := []ui.GridSpec{
@@ -819,7 +860,10 @@ func (pv *PartViewer) Draw(ctx *ui.Context) {
 
 func (pv *PartViewer) Cleanup() {
 	if pv.pager != nil && pv.pager.Process != nil {
-		pv.pager.Process.Kill()
+		err := pv.pager.Process.Kill()
+		if err != nil {
+			logging.Warnf("failed to kill pager process: %v", err)
+		}
 		pv.pager = nil
 	}
 }
