@@ -12,6 +12,7 @@ import (
 	"github.com/syndtr/goleveldb/leveldb"
 
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -89,12 +90,12 @@ func (w *IMAPWorker) newClient(c *client.Client) {
 	sort, err := w.client.sort.SupportSort()
 	if err == nil && sort {
 		w.caps.Sort = true
-		w.worker.Logger.Println("Server Capability found: Sort")
+		logging.Infof("Server Capability found: Sort")
 	}
 	thread, err := w.client.thread.SupportThread()
 	if err == nil && thread {
 		w.caps.Thread = true
-		w.worker.Logger.Println("Server Capability found: Thread")
+		logging.Infof("Server Capability found: Thread")
 	}
 }
 
@@ -204,6 +205,8 @@ func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
 		w.handleAnsweredMessages(msg)
 	case *types.CopyMessages:
 		w.handleCopyMessages(msg)
+	case *types.MoveMessages:
+		w.handleMoveMessages(msg)
 	case *types.AppendMessage:
 		w.handleAppendMessage(msg)
 	case *types.SearchDirectory:
@@ -223,7 +226,7 @@ func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
 }
 
 func (w *IMAPWorker) handleImapUpdate(update client.Update) {
-	w.worker.Logger.Printf("(= %T", update)
+	logging.Debugf("(= %T", update)
 	switch update := update.(type) {
 	case *client.MailboxUpdate:
 		status := update.Mailbox
@@ -246,12 +249,15 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		msg := update.Message
 		if msg.Uid == 0 {
 			if uid, found := w.seqMap.Get(msg.SeqNum); !found {
-				w.worker.Logger.Printf("MessageUpdate unknown seqnum: %v", msg.SeqNum)
+				logging.Errorf("MessageUpdate unknown seqnum: %d", msg.SeqNum)
 				return
 			} else {
 				msg.Uid = uid
 			}
 		}
+		if int(msg.SeqNum) > w.seqMap.Size() {
+			w.seqMap.Put(msg.Uid)
+		}
 		w.worker.PostMessage(&types.MessageInfo{
 			Info: &models.MessageInfo{
 				BodyStructure: translateBodyStructure(msg.BodyStructure),
@@ -263,7 +269,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		}, nil)
 	case *client.ExpungeUpdate:
 		if uid, found := w.seqMap.Pop(update.SeqNum); !found {
-			w.worker.Logger.Printf("ExpungeUpdate unknown seqnum: %v", update.SeqNum)
+			logging.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
 		} else {
 			w.worker.PostMessage(&types.MessagesDeleted{
 				Uids: []uint32{uid},
@@ -278,7 +284,7 @@ func (w *IMAPWorker) Run() {
 		case msg := <-w.worker.Actions:
 			msg = w.worker.ProcessAction(msg)
 
-			if err := w.handleMessage(msg); err == errUnsupported {
+			if err := w.handleMessage(msg); errors.Is(err, errUnsupported) {
 				w.worker.PostMessage(&types.Unsupported{
 					Message: types.RespondTo(msg),
 				}, nil)
