@@ -19,7 +19,7 @@ import (
 // RFC 1123Z regexp
 var dateRe = regexp.MustCompile(`(((Mon|Tue|Wed|Thu|Fri|Sat|Sun))[,]?\s[0-9]{1,2})\s` +
 	`(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s` +
-	`([0-9]{4})\s([0-9]{2}):([0-9]{2})(:([0-9]{2}))?\s([\+|\-][0-9]{4})\s?`)
+	`([0-9]{4})\s([0-9]{2}):([0-9]{2})(:([0-9]{2}))?\s([\+|\-][0-9]{4})`)
 
 func FetchEntityPartReader(e *message.Entity, index []int) (io.Reader, error) {
 	if len(index) == 0 {
@@ -46,7 +46,7 @@ func FetchEntityPartReader(e *message.Entity, index []int) (io.Reader, error) {
 	return nil, fmt.Errorf("FetchEntityPartReader: unexpected code reached")
 }
 
-//TODO: the UI doesn't seem to like readers which aren't buffers
+// TODO: the UI doesn't seem to like readers which aren't buffers
 func bufReader(e *message.Entity) (io.Reader, error) {
 	var buf bytes.Buffer
 	if _, err := io.Copy(&buf, e.Body); err != nil {
@@ -64,12 +64,37 @@ func splitMIME(m string) (string, string) {
 	return parts[0], parts[1]
 }
 
+func fixContentType(h message.Header) (string, map[string]string) {
+	ct, rest := h.Get("Content-Type"), ""
+	if i := strings.Index(ct, ";"); i > 0 {
+		ct, rest = ct[:i], ct[i:]
+	}
+
+	// check if there are quotes around the content type
+	if strings.Contains(ct, "\"") {
+		header := strings.ReplaceAll(ct, "\"", "")
+		if rest != "" {
+			header += rest
+		}
+		h.Set("Content-Type", header)
+		if contenttype, params, err := h.ContentType(); err == nil {
+			return contenttype, params
+		}
+	}
+
+	// if all else fails, return text/plain
+	return "text/plain", nil
+}
+
 func ParseEntityStructure(e *message.Entity) (*models.BodyStructure, error) {
 	var body models.BodyStructure
 	contentType, ctParams, err := e.Header.ContentType()
 	if err != nil {
-		return nil, fmt.Errorf("could not parse content type: %v", err)
+		// try to fix the error; if all measures fail, then return a
+		// text/plain content type to display at least plaintext
+		contentType, ctParams = fixContentType(e.Header)
 	}
+
 	mimeType, mimeSubType := splitMIME(contentType)
 	body.MIMEType = mimeType
 	body.MIMESubType = mimeSubType
@@ -79,7 +104,7 @@ func ParseEntityStructure(e *message.Entity) (*models.BodyStructure, error) {
 	if cd := e.Header.Get("content-disposition"); cd != "" {
 		contentDisposition, cdParams, err := e.Header.ContentDisposition()
 		if err != nil {
-			return nil, fmt.Errorf("could not parse content disposition: %v", err)
+			return nil, fmt.Errorf("could not parse content disposition: %w", err)
 		}
 		body.Disposition = contentDisposition
 		body.DispositionParams = cdParams
@@ -88,7 +113,7 @@ func ParseEntityStructure(e *message.Entity) (*models.BodyStructure, error) {
 	if mpr := e.MultipartReader(); mpr != nil {
 		for {
 			part, err := mpr.NextPart()
-			if err == io.EOF {
+			if errors.Is(err, io.EOF) {
 				return &body, nil
 			} else if err != nil {
 				return nil, err
@@ -108,31 +133,31 @@ var DateParseError = errors.New("date parsing failed")
 func parseEnvelope(h *mail.Header) (*models.Envelope, error) {
 	from, err := parseAddressList(h, "from")
 	if err != nil {
-		return nil, fmt.Errorf("could not read from address: %v", err)
+		return nil, fmt.Errorf("could not read from address: %w", err)
 	}
 	to, err := parseAddressList(h, "to")
 	if err != nil {
-		return nil, fmt.Errorf("could not read to address: %v", err)
+		return nil, fmt.Errorf("could not read to address: %w", err)
 	}
 	cc, err := parseAddressList(h, "cc")
 	if err != nil {
-		return nil, fmt.Errorf("could not read cc address: %v", err)
+		return nil, fmt.Errorf("could not read cc address: %w", err)
 	}
 	bcc, err := parseAddressList(h, "bcc")
 	if err != nil {
-		return nil, fmt.Errorf("could not read bcc address: %v", err)
+		return nil, fmt.Errorf("could not read bcc address: %w", err)
 	}
 	replyTo, err := parseAddressList(h, "reply-to")
 	if err != nil {
-		return nil, fmt.Errorf("could not read reply-to address: %v", err)
+		return nil, fmt.Errorf("could not read reply-to address: %w", err)
 	}
 	subj, err := h.Subject()
 	if err != nil {
-		return nil, fmt.Errorf("could not read subject: %v", err)
+		return nil, fmt.Errorf("could not read subject: %w", err)
 	}
 	msgID, err := h.MessageID()
 	if err != nil {
-		//proper parsing failed, so fall back to whatever is there
+		// proper parsing failed, so fall back to whatever is there
 		msgID, err = h.Text("message-id")
 		if err != nil {
 			return nil, err
@@ -192,7 +217,7 @@ func parseDate(h *mail.Header) (time.Time, error) {
 func parseReceivedHeader(h *mail.Header) (time.Time, error) {
 	guess, err := h.Text("received")
 	if err != nil {
-		return time.Time{}, fmt.Errorf("received header not parseable: %v",
+		return time.Time{}, fmt.Errorf("received header not parseable: %w",
 			err)
 	}
 	return time.Parse(time.RFC1123Z, dateRe.FindString(guess))
@@ -229,18 +254,18 @@ func MessageInfo(raw RawMessage) (*models.MessageInfo, error) {
 	defer r.Close()
 	msg, err := message.Read(r)
 	if err != nil {
-		return nil, fmt.Errorf("could not read message: %v", err)
+		return nil, fmt.Errorf("could not read message: %w", err)
 	}
 	bs, err := ParseEntityStructure(msg)
 	if errors.As(err, new(message.UnknownEncodingError)) {
 		parseErr = err
 	} else if err != nil {
-		return nil, fmt.Errorf("could not get structure: %v", err)
+		return nil, fmt.Errorf("could not get structure: %w", err)
 	}
 	h := &mail.Header{Header: msg.Header}
 	env, err := parseEnvelope(h)
 	if err != nil && !errors.Is(err, DateParseError) {
-		return nil, fmt.Errorf("could not parse envelope: %v", err)
+		return nil, fmt.Errorf("could not parse envelope: %w", err)
 		// if only the date parsing failed we still get the rest of the
 		// envelop structure in a valid state.
 		// Date parsing errors are fairly common and it's better to be
