@@ -1,7 +1,6 @@
 package maildir
 
 import (
-	"bufio"
 	"fmt"
 	"io"
 
@@ -20,12 +19,8 @@ type Message struct {
 }
 
 // NewReader reads a message into memory and returns an io.Reader for it.
-func (m Message) NewReader() (io.Reader, error) {
-	f, err := m.dir.Open(m.key)
-	if err != nil {
-		return nil, err
-	}
-	return bufio.NewReader(f), nil
+func (m Message) NewReader() (io.ReadCloser, error) {
+	return m.dir.Open(m.key)
 }
 
 // Flags fetches the set of flags currently applied to the message.
@@ -51,7 +46,7 @@ func (m Message) SetFlags(flags []maildir.Flag) error {
 func (m Message) SetOneFlag(flag maildir.Flag, enable bool) error {
 	flags, err := m.Flags()
 	if err != nil {
-		return fmt.Errorf("could not read previous flags: %v", err)
+		return fmt.Errorf("could not read previous flags: %w", err)
 	}
 	if enable {
 		flags = append(flags, flag)
@@ -92,7 +87,7 @@ func (m Message) NewBodyPartReader(requestedParts []int) (io.Reader, error) {
 	defer f.Close()
 	msg, err := message.Read(f)
 	if err != nil {
-		return nil, fmt.Errorf("could not read message: %v", err)
+		return nil, fmt.Errorf("could not read message: %w", err)
 	}
 	return lib.FetchEntityPartReader(msg, requestedParts)
 }
