@@ -6,16 +6,18 @@ package notmuch
 import (
 	"bufio"
 	"bytes"
+	"context"
+	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strings"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
-	"git.sr.ht/~rjarry/aerc/lib/uidstore"
 	"git.sr.ht/~rjarry/aerc/logging"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
@@ -38,7 +40,6 @@ type worker struct {
 	nmEvents            chan eventType
 	query               string
 	currentQueryName    string
-	uidStore            *uidstore.Store
 	queryMapOrder       []string
 	nameQueryMap        map[string]string
 	db                  *notmuch.DB
@@ -49,8 +50,10 @@ type worker struct {
 // NewWorker creates a new notmuch worker with the provided worker.
 func NewWorker(w *types.Worker) (types.Backend, error) {
 	events := make(chan eventType, 20)
-	return &worker{w: w,
-		nmEvents: events}, nil
+	return &worker{
+		w:        w,
+		nmEvents: events,
+	}, nil
 }
 
 // Run starts the worker's message handling loop.
@@ -59,29 +62,29 @@ func (w *worker) Run() {
 		select {
 		case action := <-w.w.Actions:
 			msg := w.w.ProcessAction(action)
-			if err := w.handleMessage(msg); err == errUnsupported {
+			if err := w.handleMessage(msg); errors.Is(err, errUnsupported) {
 				w.w.PostMessage(&types.Unsupported{
 					Message: types.RespondTo(msg),
 				}, nil)
-				w.w.Logger.Printf("ProcessAction(%T) unsupported: %v", msg, err)
+				logging.Errorf("ProcessAction(%T) unsupported: %w", msg, err)
 			} else if err != nil {
 				w.w.PostMessage(&types.Error{
 					Message: types.RespondTo(msg),
 					Error:   err,
 				}, nil)
-				w.w.Logger.Printf("ProcessAction(%T) failure: %v", msg, err)
+				logging.Errorf("ProcessAction(%T) failure: %w", msg, err)
 			}
 		case nmEvent := <-w.nmEvents:
 			err := w.handleNotmuchEvent(nmEvent)
 			if err != nil {
-				w.w.Logger.Printf("notmuch event failure: %v", err)
+				logging.Errorf("notmuch event failure: %w", err)
 			}
 		}
 	}
 }
 
 func (w *worker) done(msg types.WorkerMessage) {
-	w.w.PostMessage(&types.Done{types.RespondTo(msg)}, nil)
+	w.w.PostMessage(&types.Done{Message: types.RespondTo(msg)}, nil)
 }
 
 func (w *worker) err(msg types.WorkerMessage, err error) {
@@ -90,6 +93,7 @@ func (w *worker) err(msg types.WorkerMessage, err error) {
 		Error:   err,
 	}, nil)
 }
+
 func (w *worker) handleMessage(msg types.WorkerMessage) error {
 	if w.setupErr != nil {
 		// only configure can recover from a config error, bail for everything else
@@ -128,6 +132,9 @@ func (w *worker) handleMessage(msg types.WorkerMessage) error {
 		return w.handleSearchDirectory(msg)
 	case *types.ModifyLabels:
 		return w.handleModifyLabels(msg)
+	case *types.CheckMail:
+		go w.handleCheckMail(msg)
+		return nil
 
 		// not implemented, they are generally not used
 		// in a notmuch based workflow
@@ -151,25 +158,25 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 			w.setupErr = nil
 			return
 		}
-		w.setupErr = fmt.Errorf("notmuch: %v", err)
+		w.setupErr = fmt.Errorf("notmuch: %w", err)
 	}()
 
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		w.w.Logger.Printf("error configuring notmuch worker: %v", err)
+		logging.Errorf("error configuring notmuch worker: %w", err)
 		return err
 	}
 	home, err := homedir.Expand(u.Hostname())
 	if err != nil {
-		return fmt.Errorf("could not resolve home directory: %v", err)
+		return fmt.Errorf("could not resolve home directory: %w", err)
 	}
 	pathToDB := filepath.Join(home, u.Path)
 	err = w.loadQueryMap(msg.Config)
 	if err != nil {
-		return fmt.Errorf("could not load query map configuration: %v", err)
+		return fmt.Errorf("could not load query map configuration: %w", err)
 	}
 	excludedTags := w.loadExcludeTags(msg.Config)
-	w.db = notmuch.NewDB(pathToDB, excludedTags, w.w.Logger)
+	w.db = notmuch.NewDB(pathToDB, excludedTags)
 	return nil
 }
 
@@ -206,12 +213,20 @@ func (w *worker) handleListDirectories(msg *types.ListDirectories) error {
 }
 
 func (w *worker) gatherDirectoryInfo(name string, query string) (
-	*types.DirectoryInfo, error) {
+	*types.DirectoryInfo, error,
+) {
+	return w.buildDirInfo(name, query, false)
+}
+
+func (w *worker) buildDirInfo(name string, query string, skipSort bool) (
+	*types.DirectoryInfo, error,
+) {
 	count, err := w.db.QueryCountMessages(query)
 	if err != nil {
 		return nil, err
 	}
 	info := &types.DirectoryInfo{
+		SkipSort: skipSort,
 		Info: &models.DirectoryInfo{
 			Name:     name,
 			Flags:    []string{},
@@ -223,6 +238,11 @@ func (w *worker) gatherDirectoryInfo(name string, query string) (
 			// total unread
 			Unseen:         count.Unread,
 			AccurateCounts: true,
+
+			Caps: &models.Capabilities{
+				Sort:   true,
+				Thread: true,
+			},
 		},
 	}
 	return info, nil
@@ -238,8 +258,8 @@ func (w *worker) emitDirectoryInfo(name string) error {
 	return nil
 }
 
-//queryFromName either returns the friendly ID if aliased or the name itself
-//assuming it to be the query
+// queryFromName either returns the friendly ID if aliased or the name itself
+// assuming it to be the query
 func (w *worker) queryFromName(name string) string {
 	// try the friendly name first, if that fails assume it's a query
 	q, ok := w.nameQueryMap[name]
@@ -250,7 +270,7 @@ func (w *worker) queryFromName(name string) string {
 }
 
 func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	w.w.Logger.Printf("opening %s", msg.Directory)
+	logging.Infof("opening %s", msg.Directory)
 	// try the friendly name first, if that fails assume it's a query
 	w.query = w.queryFromName(msg.Directory)
 	w.currentQueryName = msg.Directory
@@ -265,7 +285,8 @@ func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
 }
 
 func (w *worker) handleFetchDirectoryContents(
-	msg *types.FetchDirectoryContents) error {
+	msg *types.FetchDirectoryContents,
+) error {
 	w.currentSortCriteria = msg.SortCriteria
 	err := w.emitDirectoryContents(msg)
 	if err != nil {
@@ -276,7 +297,8 @@ func (w *worker) handleFetchDirectoryContents(
 }
 
 func (w *worker) handleFetchDirectoryThreaded(
-	msg *types.FetchDirectoryThreaded) error {
+	msg *types.FetchDirectoryThreaded,
+) error {
 	// w.currentSortCriteria = msg.SortCriteria
 	err := w.emitDirectoryThreaded(msg)
 	if err != nil {
@@ -287,17 +309,18 @@ func (w *worker) handleFetchDirectoryThreaded(
 }
 
 func (w *worker) handleFetchMessageHeaders(
-	msg *types.FetchMessageHeaders) error {
+	msg *types.FetchMessageHeaders,
+) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			w.w.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			w.w.Logger.Printf(err.Error())
+			logging.Errorf("could not emit message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
@@ -334,17 +357,17 @@ func (w *worker) msgFromUid(uid uint32) (*Message, error) {
 }
 
 func (w *worker) handleFetchMessageBodyPart(
-	msg *types.FetchMessageBodyPart) error {
-
+	msg *types.FetchMessageBodyPart,
+) error {
 	m, err := w.msgFromUid(msg.Uid)
 	if err != nil {
-		w.w.Logger.Printf("could not get message %d: %v", msg.Uid, err)
+		logging.Errorf("could not get message %d: %w", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		w.w.Logger.Printf(
-			"could not get body part reader for message=%d, parts=%#v: %v",
+		logging.Errorf(
+			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
 	}
@@ -364,16 +387,16 @@ func (w *worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			w.w.Logger.Printf("could not get message %d: %v", uid, err)
+			logging.Errorf("could not get message %d: %w", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			w.w.Logger.Printf("could not get message reader: %v", err)
+			logging.Errorf("could not get message reader: %w", err)
 			return err
 		}
 		defer r.Close()
-		b, err := ioutil.ReadAll(r)
+		b, err := io.ReadAll(r)
 		if err != nil {
 			return err
 		}
@@ -393,24 +416,24 @@ func (w *worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			w.w.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkAnswered(msg.Answered); err != nil {
-			w.w.Logger.Printf("could not mark message as answered: %v", err)
+			logging.Errorf("could not mark message as answered: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			w.w.Logger.Printf(err.Error())
+			logging.Errorf("could not emit message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
 	}
 	if err := w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		w.w.Logger.Printf(err.Error())
+		logging.Errorf("could not emit directory info: %w", err)
 	}
 	w.done(msg)
 	return nil
@@ -420,24 +443,24 @@ func (w *worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			w.w.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.SetFlag(msg.Flag, msg.Enable); err != nil {
-			w.w.Logger.Printf("could not set flag %v as %v for message: %v", msg.Flag, msg.Enable, err)
+			logging.Errorf("could not set flag %v as %t for message: %w", msg.Flag, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			w.w.Logger.Printf(err.Error())
+			logging.Errorf("could not emit message info: %w", err)
 			w.err(msg, err)
 			continue
 		}
 	}
 	if err := w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		w.w.Logger.Printf(err.Error())
+		logging.Errorf("could not emit directory info: %w", err)
 	}
 	w.done(msg)
 	return nil
@@ -466,11 +489,11 @@ func (w *worker) handleModifyLabels(msg *types.ModifyLabels) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			return fmt.Errorf("could not get message from uid %v: %v", uid, err)
+			return fmt.Errorf("could not get message from uid %d: %w", uid, err)
 		}
 		err = m.ModifyTags(msg.Add, msg.Remove)
 		if err != nil {
-			return fmt.Errorf("could not modify message tags: %v", err)
+			return fmt.Errorf("could not modify message tags: %w", err)
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
@@ -486,7 +509,7 @@ func (w *worker) handleModifyLabels(msg *types.ModifyLabels) error {
 	// and update the list of possible tags
 	w.emitLabelList()
 	if err = w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		w.w.Logger.Printf(err.Error())
+		logging.Errorf("could not emit directory info: %w", err)
 	}
 	w.done(msg)
 	return nil
@@ -526,7 +549,8 @@ func (w *worker) loadQueryMap(acctConfig *config.AccountConfig) error {
 }
 
 func (w *worker) loadExcludeTags(
-	acctConfig *config.AccountConfig) []string {
+	acctConfig *config.AccountConfig,
+) []string {
 	raw, ok := acctConfig.Params["exclude-tags"]
 	if !ok {
 		// nothing to do
@@ -540,13 +564,20 @@ func (w *worker) loadExcludeTags(
 }
 
 func (w *worker) emitDirectoryContents(parent types.WorkerMessage) error {
-	uids, err := w.uidsFromQuery(w.query)
+	query := w.query
+	if msg, ok := parent.(*types.FetchDirectoryContents); ok {
+		s := strings.Join(msg.FilterCriteria[1:], " ")
+		if s != "" {
+			query = fmt.Sprintf("(%v) and (%v)", query, s)
+		}
+	}
+	uids, err := w.uidsFromQuery(query)
 	if err != nil {
-		return fmt.Errorf("could not fetch uids: %v", err)
+		return fmt.Errorf("could not fetch uids: %w", err)
 	}
 	sortedUids, err := w.sort(uids, w.currentSortCriteria)
 	if err != nil {
-		w.w.Logger.Printf("error sorting directory: %v", err)
+		logging.Errorf("error sorting directory: %w", err)
 		return err
 	}
 	w.w.PostMessage(&types.DirectoryContents{
@@ -557,22 +588,29 @@ func (w *worker) emitDirectoryContents(parent types.WorkerMessage) error {
 }
 
 func (w *worker) emitDirectoryThreaded(parent types.WorkerMessage) error {
-	threads, err := w.db.ThreadsFromQuery(w.query)
+	query := w.query
+	if msg, ok := parent.(*types.FetchDirectoryThreaded); ok {
+		s := strings.Join(msg.FilterCriteria[1:], " ")
+		if s != "" {
+			query = fmt.Sprintf("(%v) and (%v)", query, s)
+		}
+	}
+	threads, err := w.db.ThreadsFromQuery(query)
 	if err != nil {
 		return err
 	}
 	w.w.PostMessage(&types.DirectoryThreaded{
-		Message: types.RespondTo(parent),
 		Threads: threads,
 	}, nil)
 	return nil
 }
 
 func (w *worker) emitMessageInfo(m *Message,
-	parent types.WorkerMessage) error {
+	parent types.WorkerMessage,
+) error {
 	info, err := m.MessageInfo()
 	if err != nil {
-		return fmt.Errorf("could not get MessageInfo: %v", err)
+		return fmt.Errorf("could not get MessageInfo: %w", err)
 	}
 	w.w.PostMessage(&types.MessageInfo{
 		Message: types.RespondTo(parent),
@@ -584,14 +622,15 @@ func (w *worker) emitMessageInfo(m *Message,
 func (w *worker) emitLabelList() {
 	tags, err := w.db.ListTags()
 	if err != nil {
-		w.w.Logger.Printf("could not load tags: %v", err)
+		logging.Errorf("could not load tags: %w", err)
 		return
 	}
 	w.w.PostMessage(&types.LabelList{Labels: tags}, nil)
 }
 
 func (w *worker) sort(uids []uint32,
-	criteria []*types.SortCriterion) ([]uint32, error) {
+	criteria []*types.SortCriterion,
+) ([]uint32, error) {
 	if len(criteria) == 0 {
 		return uids, nil
 	}
@@ -599,20 +638,45 @@ func (w *worker) sort(uids []uint32,
 	for _, uid := range uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			w.w.Logger.Printf("could not get message: %v", err)
+			logging.Errorf("could not get message: %w", err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			w.w.Logger.Printf("could not get message info: %v", err)
+			logging.Errorf("could not get message info: %w", err)
 			continue
 		}
 		msgInfos = append(msgInfos, info)
 	}
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		w.w.Logger.Printf("could not sort the messages: %v", err)
+		logging.Errorf("could not sort the messages: %w", err)
 		return nil, err
 	}
 	return sortedUids, nil
 }
+
+func (w *worker) handleCheckMail(msg *types.CheckMail) {
+	if msg.Command == "" {
+		w.err(msg, fmt.Errorf("checkmail: no command specified"))
+		return
+	}
+	ctx, cancel := context.WithTimeout(context.Background(), msg.Timeout)
+	defer cancel()
+	cmd := exec.CommandContext(ctx, "sh", "-c", msg.Command)
+	ch := make(chan error)
+	go func() {
+		err := cmd.Run()
+		ch <- err
+	}()
+	select {
+	case <-ctx.Done():
+		w.err(msg, fmt.Errorf("checkmail: timed out"))
+	case err := <-ch:
+		if err != nil {
+			w.err(msg, fmt.Errorf("checkmail: error running command: %w", err))
+		} else {
+			w.done(msg)
+		}
+	}
+}
