@@ -165,6 +165,12 @@ type CopyMessages struct {
 	Uids        []uint32
 }
 
+type MoveMessages struct {
+	Message
+	Destination string
+	Uids        []uint32
+}
+
 type AppendMessage struct {
 	Message
 	Destination string
@@ -231,6 +237,12 @@ type MessagesCopied struct {
 	Uids        []uint32
 }
 
+type MessagesMoved struct {
+	Message
+	Destination string
+	Uids        []uint32
+}
+
 type ModifyLabels struct {
 	Message
 	Uids   []uint32
